@@ -3,6 +3,10 @@
 eltype(::Type{<:AbstractSet{T}}) where {T} = @isdefined(T) ? T : Any
 sizehint!(s::AbstractSet, n) = nothing
 
+copy!(dst::AbstractSet, src::AbstractSet) = union!(empty!(dst), src)
+
+## set operations (union, intersection, symmetric difference)
+
 """
     union(s, itrs...)
     ∪(s, itrs...)
@@ -31,7 +35,10 @@ julia> union([4, 2], 1:2)
  1
 
 julia> union(Set([1, 2]), 2:3)
-Set([2, 3, 1])
+Set{Int64} with 3 elements:
+  2
+  3
+  1
 ```
 """
 function union end
@@ -56,7 +63,12 @@ julia> a = Set([1, 3, 4, 5]);
 julia> union!(a, 1:2:8);
 
 julia> a
-Set([7, 4, 3, 5, 1])
+Set{Int64} with 5 elements:
+  7
+  4
+  3
+  5
+  1
 ```
 """
 function union!(s::AbstractSet, sets...)
@@ -75,11 +87,11 @@ max_values(::Type{Nothing}) = 1
 
 function union!(s::AbstractSet{T}, itr) where T
     haslength(itr) && sizehint!(s, length(s) + length(itr))
-    for x=itr
+    for x in itr
         push!(s, x)
         length(s) == max_values(T) && break
     end
-    s
+    return s
 end
 
 """
@@ -101,7 +113,8 @@ julia> intersect([1, 4, 4, 5, 6], [4, 6, 6, 7, 8])
  6
 
 julia> intersect(Set([1, 2]), BitSet([2, 3]))
-Set([2])
+Set{Int64} with 1 element:
+  2
 ```
 """
 intersect(s::AbstractSet, itr, itrs...) = intersect!(intersect(s, itr), itrs...)
@@ -156,7 +169,8 @@ julia> a = Set([1, 3, 4, 5]);
 julia> setdiff!(a, 1:2:6);
 
 julia> a
-Set([4])
+Set{Int64} with 1 element:
+  4
 ```
 """
 function setdiff!(s::AbstractSet, itrs...)
@@ -218,41 +232,16 @@ function symdiff!(s::AbstractSet, itr)
     for x in itr
         x in s ? delete!(s, x) : push!(s, x)
     end
-    s
+    return s
 end
 
-==(l::AbstractSet, r::AbstractSet) = length(l) == length(r) && l ⊆ r
-# convenience functions for AbstractSet
-# (if needed, only their synonyms ⊊ and ⊆ must be specialized)
-<( l::AbstractSet, r::AbstractSet) = l ⊊ r
-<=(l::AbstractSet, r::AbstractSet) = l ⊆ r
+## non-strict subset comparison
 
-function issubset(l, r)
-    if haslength(r)
-        rlen = length(r)
-        #This threshold was empirically determined by repeatedly
-        #sampling using these two methods (see #26198)
-        lenthresh = 70
-
-        if rlen > lenthresh && !isa(r, AbstractSet)
-            return issubset(l, Set(r))
-        end
-    end
-
-    for elt in l
-        if !in(elt, r)
-            return false
-        end
-    end
-    return true
-end
-# use the implementation below when it becomes as efficient
-# issubset(l, r) = all(_in(r), l)
 const ⊆ = issubset
-⊇(l, r) = r ⊆ l
+function ⊇ end
 """
-    issubset(a, b)
-    ⊆(a,b)  -> Bool
+    issubset(a, b) -> Bool
+    ⊆(a, b) -> Bool
     ⊇(b, a) -> Bool
 
 Determine whether every element of `a` is also in `b`, using [`in`](@ref).
@@ -271,29 +260,47 @@ true
 """
 issubset, ⊆, ⊇
 
+function issubset(l, r)
+    if haslength(r) && (isa(l, AbstractSet) || !hasfastin(r))
+        rlen = length(r) # conditions above make this length computed only when needed
+        # check l for too many unique elements
+        if isa(l, AbstractSet) && length(l) > rlen
+            return false
+        end
+        # when `in` would be too slow and r is big enough, convert it to a Set
+        # this threshold was empirically determined (cf. #26198)
+        if !hasfastin(r) && rlen > 70
+            return issubset(l, Set(r))
+        end
+    end
+    for elt in l
+        elt in r || return false
+    end
+    return true
+end
+
 """
-    issetequal(a, b)
+    hasfastin(T)
 
-Determine whether `a` and `b` have the same elements. Equivalent
-to `a ⊆ b && b ⊆ a`.
-
-# Examples
-```jldoctest
-julia> issetequal([1, 2], [1, 2, 3])
-false
-
-julia> issetequal([1, 2], [2, 1])
-true
-```
+Determine whether the computation `x ∈ collection` where `collection::T` can be considered
+as a "fast" operation (typically constant or logarithmic complexity).
+The definition `hasfastin(x) = hasfastin(typeof(x))` is provided for convenience so that instances
+can be passed instead of types.
+However the form that accepts a type argument should be defined for new types.
 """
-issetequal(l, r) = length(l) == length(r) && l ⊆ r
-issetequal(l::AbstractSet, r::AbstractSet) = l == r
+hasfastin(::Type) = false
+hasfastin(::Union{Type{<:AbstractSet},Type{<:AbstractDict},Type{<:AbstractRange}}) = true
+hasfastin(x) = hasfastin(typeof(x))
 
-⊊(l, r) = length(l) < length(r) && l ⊆ r
-⊋(l, r) = r ⊊ l
+⊇(l, r) = r ⊆ l
+
+## strict subset comparison
+
+function ⊊ end
+function ⊋ end
 """
-    ⊊(a, b)
-    ⊋(b, a)
+    ⊊(a, b) -> Bool
+    ⊋(b, a) -> Bool
 
 Determines if `a` is a subset of, but not equal to, `b`.
 
@@ -308,11 +315,15 @@ false
 """
 ⊊, ⊋
 
-⊈(l, r) = !⊆(l, r)
-⊉(l, r) = r ⊈ l
+⊊(l::AbstractSet, r) = length(l) < length(r) && l ⊆ r
+⊊(l, r) = Set(l) ⊊ r
+⊋(l, r) = r ⊊ l
+
+function ⊈ end
+function ⊉ end
 """
-    ⊈(a, b)
-    ⊉(b, a)
+    ⊈(a, b) -> Bool
+    ⊉(b, a) -> Bool
 
 Negation of `⊆` and `⊇`, i.e. checks that `a` is not a subset of `b`.
 
@@ -327,6 +338,53 @@ false
 """
 ⊈, ⊉
 
+⊈(l, r) = !⊆(l, r)
+⊉(l, r) = r ⊈ l
+
+## set equality comparison
+
+"""
+    issetequal(a, b) -> Bool
+
+Determine whether `a` and `b` have the same elements. Equivalent
+to `a ⊆ b && b ⊆ a` but more efficient when possible.
+
+# Examples
+```jldoctest
+julia> issetequal([1, 2], [1, 2, 3])
+false
+
+julia> issetequal([1, 2], [2, 1])
+true
+```
+"""
+issetequal(l::AbstractSet, r::AbstractSet) = l == r
+issetequal(l::AbstractSet, r) = issetequal(l, Set(r))
+
+function issetequal(l, r::AbstractSet)
+    if haslength(l)
+        # check r for too many unique elements
+        length(l) < length(r) && return false
+    end
+    return issetequal(Set(l), r)
+end
+
+function issetequal(l, r)
+    haslength(l) && return issetequal(l, Set(r))
+    haslength(r) && return issetequal(r, Set(l))
+    return issetequal(Set(l), Set(r))
+end
+
+## partial ordering of sets by containment
+
+==(l::AbstractSet, r::AbstractSet) = length(l) == length(r) && l ⊆ r
+# convenience functions for AbstractSet
+# (if needed, only their synonyms ⊊ and ⊆ must be specialized)
+<( l::AbstractSet, r::AbstractSet) = l ⊊ r
+<=(l::AbstractSet, r::AbstractSet) = l ⊆ r
+
+## filtering sets
+
 filter(pred, s::AbstractSet) = mapfilter(pred, push!, s, emptymutable(s))
 
 # it must be safe to delete the current element while iterating over s:
