@@ -26,7 +26,7 @@ and will be converted automatically at the call site to the appropriate type.
 
 See [`@cfunction`](@ref).
 """
-struct CFunction <: Ref{Cvoid}
+mutable struct CFunction <: Ref{Cvoid}
     ptr::Ptr{Cvoid}
     f::Any
     _1::Ptr{Cvoid}
@@ -39,7 +39,7 @@ unsafe_convert(::Type{Ptr{Cvoid}}, cf::CFunction) = cf.ptr
     @cfunction(callable, ReturnType, (ArgumentTypes...,)) -> Ptr{Cvoid}
     @cfunction(\$callable, ReturnType, (ArgumentTypes...,)) -> CFunction
 
-Generate a C-callable function pointer from the Julia function `closure`
+Generate a C-callable function pointer from the Julia function `callable`
 for the given type signature.
 To pass the return value to a `ccall`, use the argument type `Ptr{Cvoid}` in the signature.
 
@@ -47,7 +47,7 @@ Note that the argument type tuple must be a literal tuple, and not a tuple-value
 (although it can include a splat expression). And that these arguments will be evaluated in global scope
 during compile-time (not deferred until runtime).
 Adding a '\\\$' in front of the function argument changes this to instead create a runtime closure
-over the local variable `callable`.
+over the local variable `callable` (this is not supported on all architectures).
 
 See [manual section on ccall and cfunction usage](@ref Calling-C-and-Fortran-Code).
 
@@ -61,12 +61,12 @@ julia> @cfunction(foo, Int, (Int, Int))
 Ptr{Cvoid} @0x000000001b82fcd0
 ```
 """
-macro cfunction(f, at, rt)
-    if !(isa(rt, Expr) && rt.head === :tuple)
+macro cfunction(f, rt, at)
+    if !(isa(at, Expr) && at.head === :tuple)
         throw(ArgumentError("@cfunction argument types must be a literal tuple"))
     end
-    rt.head = :call
-    pushfirst!(rt.args, GlobalRef(Core, :svec))
+    at.head = :call
+    pushfirst!(at.args, GlobalRef(Core, :svec))
     if isa(f, Expr) && f.head === :$
         fptr = f.args[1]
         typ = CFunction
@@ -74,7 +74,7 @@ macro cfunction(f, at, rt)
         fptr = QuoteNode(f)
         typ = Ptr{Cvoid}
     end
-    cfun = Expr(:cfunction, typ, fptr, at, rt, QuoteNode(:ccall))
+    cfun = Expr(:cfunction, typ, fptr, rt, at, QuoteNode(:ccall))
     return esc(cfun)
 end
 
@@ -183,7 +183,7 @@ Calling [`Ref(array[, index])`](@ref Ref) is generally preferable to this functi
 """
 function pointer end
 
-pointer(p::Cstring) = convert(Ptr{UInt8}, p)
+pointer(p::Cstring) = convert(Ptr{Cchar}, p)
 pointer(p::Cwstring) = convert(Ptr{Cwchar_t}, p)
 
 # comparisons against pointers (mainly to support `cstr==C_NULL`)
@@ -203,7 +203,7 @@ function cconvert(::Type{Cwstring}, s::AbstractString)
     return v
 end
 
-eltype(::Type{Cstring}) = UInt8
+eltype(::Type{Cstring}) = Cchar
 eltype(::Type{Cwstring}) = Cwchar_t
 
 containsnul(p::Ptr, len) =
@@ -292,7 +292,7 @@ transcode(T, src::String) = transcode(T, codeunits(src))
 transcode(::Type{String}, src) = String(transcode(UInt8, src))
 
 function transcode(::Type{UInt16}, src::AbstractVector{UInt8})
-    @assert !has_offset_axes(src)
+    require_one_based_indexing(src)
     dst = UInt16[]
     i, n = 1, length(src)
     n > 0 || return dst
@@ -343,7 +343,7 @@ function transcode(::Type{UInt16}, src::AbstractVector{UInt8})
 end
 
 function transcode(::Type{UInt8}, src::AbstractVector{UInt16})
-    @assert !has_offset_axes(src)
+    require_one_based_indexing(src)
     n = length(src)
     n == 0 && return UInt8[]
 
