@@ -22,9 +22,6 @@ mutable struct Regex
     compile_options::UInt32
     match_options::UInt32
     regex::Ptr{Cvoid}
-    extra::Ptr{Cvoid}
-    ovec::Vector{Csize_t}
-    match_data::Ptr{Cvoid}
 
     function Regex(pattern::AbstractString, compile_options::Integer,
                    match_options::Integer)
@@ -37,11 +34,9 @@ mutable struct Regex
         if (match_options & ~PCRE.EXECUTE_MASK) !=0
             throw(ArgumentError("invalid regex match options: $match_options"))
         end
-        re = compile(new(pattern, compile_options, match_options, C_NULL,
-                         C_NULL, Csize_t[], C_NULL))
+        re = compile(new(pattern, compile_options, match_options, C_NULL))
         finalizer(re) do re
             re.regex == C_NULL || PCRE.free_re(re.regex)
-            re.match_data == C_NULL || PCRE.free_match_data(re.match_data)
         end
         re
     end
@@ -66,10 +61,21 @@ Regex(pattern::AbstractString) = Regex(pattern, DEFAULT_COMPILER_OPTS, DEFAULT_M
 
 function compile(regex::Regex)
     if regex.regex == C_NULL
-        regex.regex = PCRE.compile(regex.pattern, regex.compile_options)
-        PCRE.jit_compile(regex.regex)
-        regex.match_data = PCRE.create_match_data(regex.regex)
-        regex.ovec = PCRE.get_ovec(regex.match_data)
+        if PCRE.PCRE_COMPILE_LOCK === nothing
+            regex.regex = PCRE.compile(regex.pattern, regex.compile_options)
+            PCRE.jit_compile(regex.regex)
+        else
+            l = PCRE.PCRE_COMPILE_LOCK::Threads.SpinLock
+            lock(l)
+            try
+                if regex.regex == C_NULL
+                    regex.regex = PCRE.compile(regex.pattern, regex.compile_options)
+                    PCRE.jit_compile(regex.regex)
+                end
+            finally
+                unlock(l)
+            end
+        end
     end
     regex
 end
@@ -164,14 +170,76 @@ getindex(m::RegexMatch, name::AbstractString) = m[Symbol(name)]
 
 function occursin(r::Regex, s::AbstractString; offset::Integer=0)
     compile(r)
-    return PCRE.exec(r.regex, String(s), offset, r.match_options,
-                     r.match_data)
+    return PCRE.exec_r(r.regex, String(s), offset, r.match_options)
 end
 
 function occursin(r::Regex, s::SubString; offset::Integer=0)
     compile(r)
-    return PCRE.exec(r.regex, s, offset, r.match_options,
-                     r.match_data)
+    return PCRE.exec_r(r.regex, s, offset, r.match_options)
+end
+
+"""
+    startswith(s::AbstractString, prefix::Regex)
+
+Return `true` if `s` starts with the regex pattern, `prefix`.
+
+!!! note
+    `startswith` does not compile the anchoring into the regular
+    expression, but instead passes the anchoring as
+    `match_option` to PCRE. If compile time is amortized,
+    `occursin(r"^...", s)` is faster than `startswith(s, r"...")`.
+
+See also [`occursin`](@ref) and [`endswith`](@ref).
+
+!!! compat "Julia 1.2"
+     This method requires at least Julia 1.2.
+
+# Examples
+```jldoctest
+julia> startswith("JuliaLang", r"Julia|Romeo")
+true
+```
+"""
+function startswith(s::AbstractString, r::Regex)
+    compile(r)
+    return PCRE.exec_r(r.regex, String(s), 0, r.match_options | PCRE.ANCHORED)
+end
+
+function startswith(s::SubString, r::Regex)
+    compile(r)
+    return PCRE.exec_r(r.regex, s, 0, r.match_options | PCRE.ANCHORED)
+end
+
+"""
+    endswith(s::AbstractString, suffix::Regex)
+
+Return `true` if `s` ends with the regex pattern, `suffix`.
+
+!!! note
+    `endswith` does not compile the anchoring into the regular
+    expression, but instead passes the anchoring as
+    `match_option` to PCRE. If compile time is amortized,
+    `occursin(r"...\$", s)` is faster than `endswith(s, r"...")`.
+
+See also [`occursin`](@ref) and [`startswith`](@ref).
+
+!!! compat "Julia 1.2"
+     This method requires at least Julia 1.2.
+
+# Examples
+```jldoctest
+julia> endswith("JuliaLang", r"Lang|Roberts")
+true
+```
+"""
+function endswith(s::AbstractString, r::Regex)
+    compile(r)
+    return PCRE.exec_r(r.regex, String(s), 0, r.match_options | PCRE.ENDANCHORED)
+end
+
+function endswith(s::SubString, r::Regex)
+    compile(r)
+    return PCRE.exec_r(r.regex, s, 0, r.match_options | PCRE.ENDANCHORED)
 end
 
 """
@@ -206,17 +274,21 @@ function match end
 function match(re::Regex, str::Union{SubString{String}, String}, idx::Integer, add_opts::UInt32=UInt32(0))
     compile(re)
     opts = re.match_options | add_opts
-    if !PCRE.exec(re.regex, str, idx-1, opts, re.match_data)
+    matched, data = PCRE.exec_r_data(re.regex, str, idx-1, opts)
+    if !matched
+        PCRE.free_match_data(data)
         return nothing
     end
-    ovec = re.ovec
-    n = div(length(ovec),2) - 1
-    mat = SubString(str, ovec[1]+1, prevind(str, ovec[2]+1))
-    cap = Union{Nothing,SubString{String}}[ovec[2i+1] == PCRE.UNSET ? nothing :
-                                        SubString(str, ovec[2i+1]+1,
-                                                  prevind(str, ovec[2i+2]+1)) for i=1:n]
-    off = Int[ ovec[2i+1]+1 for i=1:n ]
-    RegexMatch(mat, cap, ovec[1]+1, off, re)
+    n = div(PCRE.ovec_length(data), 2) - 1
+    p = PCRE.ovec_ptr(data)
+    mat = SubString(str, unsafe_load(p, 1)+1, prevind(str, unsafe_load(p, 2)+1))
+    cap = Union{Nothing,SubString{String}}[unsafe_load(p,2i+1) == PCRE.UNSET ? nothing :
+                                        SubString(str, unsafe_load(p,2i+1)+1,
+                                                  prevind(str, unsafe_load(p,2i+2)+1)) for i=1:n]
+    off = Int[ unsafe_load(p,2i+1)+1 for i=1:n ]
+    result = RegexMatch(mat, cap, unsafe_load(p,1)+1, off, re)
+    PCRE.free_match_data(data)
+    return result
 end
 
 match(r::Regex, s::AbstractString) = match(r, s, firstindex(s))
@@ -224,24 +296,92 @@ match(r::Regex, s::AbstractString, i::Integer) = throw(ArgumentError(
     "regex matching is only available for the String type; use String(s) to convert"
 ))
 
+findnext(re::Regex, str::Union{String,SubString}, idx::Integer) = _findnext_re(re, str, idx, C_NULL)
+
 # TODO: return only start index and update deprecation
-function findnext(re::Regex, str::Union{String,SubString}, idx::Integer)
+function _findnext_re(re::Regex, str::Union{String,SubString}, idx::Integer, match_data::Ptr{Cvoid})
     if idx > nextind(str,lastindex(str))
         throw(BoundsError())
     end
     opts = re.match_options
     compile(re)
-    if PCRE.exec(re.regex, str, idx-1, opts, re.match_data)
-        (Int(re.ovec[1])+1):prevind(str,Int(re.ovec[2])+1)
+    alloc = match_data == C_NULL
+    if alloc
+        matched, data = PCRE.exec_r_data(re.regex, str, idx-1, opts)
     else
-        nothing
+        matched = PCRE.exec(re.regex, str, idx-1, opts, match_data)
+        data = match_data
     end
+    if matched
+        p = PCRE.ovec_ptr(data)
+        ans = (Int(unsafe_load(p,1))+1):prevind(str,Int(unsafe_load(p,2))+1)
+    else
+        ans = nothing
+    end
+    alloc && PCRE.free_match_data(data)
+    return ans
 end
 findnext(r::Regex, s::AbstractString, idx::Integer) = throw(ArgumentError(
     "regex search is only available for the String type; use String(s) to convert"
 ))
 findfirst(r::Regex, s::AbstractString) = findnext(r,s,firstindex(s))
 
+
+"""
+    findall(
+        pattern::Union{AbstractString,Regex},
+        string::AbstractString;
+        overlap::Bool = false,
+    )
+
+Return a `Vector{UnitRange{Int}}` of all the matches for `pattern` in `string`.
+Each element of the returned vector is a range of indices where the
+matching sequence is found, like the return value of [`findnext`](@ref).
+
+If `overlap=true`, the matching sequences are allowed to overlap indices in the
+original string, otherwise they must be from disjoint character ranges.
+"""
+function findall(t::Union{AbstractString,Regex}, s::AbstractString; overlap::Bool=false)
+    found = UnitRange{Int}[]
+    i, e = firstindex(s), lastindex(s)
+    while true
+        r = findnext(t, s, i)
+        isnothing(r) && break
+        push!(found, r)
+        j = overlap || isempty(r) ? first(r) : last(r)
+        j > e && break
+        @inbounds i = nextind(s, j)
+    end
+    return found
+end
+
+"""
+    count(
+        pattern::Union{AbstractString,Regex},
+        string::AbstractString;
+        overlap::Bool = false,
+    )
+
+Return the number of matches for `pattern` in `string`. This is equivalent to
+calling `length(findall(pattern, string))` but more efficient.
+
+If `overlap=true`, the matching sequences are allowed to overlap indices in the
+original string, otherwise they must be from disjoint character ranges.
+"""
+function count(t::Union{AbstractString,Regex}, s::AbstractString; overlap::Bool=false)
+    n = 0
+    i, e = firstindex(s), lastindex(s)
+    while true
+        r = findnext(t, s, i)
+        isnothing(r) && break
+        n += 1
+        j = overlap || isempty(r) ? first(r) : last(r)
+        j > e && break
+        @inbounds i = nextind(s, j)
+    end
+    return n
+end
+
 """
     SubstitutionString(substr)
 
@@ -292,9 +432,23 @@ julia> replace(msg, r"#(.+)# from (?<from>\\w+)" => s"FROM: \\g<from>; MESSAGE:
 """
 macro s_str(string) SubstitutionString(string) end
 
+# replacement
+
+struct RegexAndMatchData
+    re::Regex
+    match_data::Ptr{Cvoid}
+    RegexAndMatchData(re::Regex) = (compile(re); new(re, PCRE.create_match_data(re.regex)))
+end
+
+findnext(pat::RegexAndMatchData, str, i) = _findnext_re(pat.re, str, i, pat.match_data)
+
+_pat_replacer(r::Regex) = RegexAndMatchData(r)
+
+_free_pat_replacer(r::RegexAndMatchData) = PCRE.free_match_data(r.match_data)
+
 replace_err(repl) = error("Bad replacement string: $repl")
 
-function _write_capture(io, re, group)
+function _write_capture(io, re::RegexAndMatchData, group)
     len = PCRE.substring_length_bynumber(re.match_data, group)
     ensureroom(io, len+1)
     PCRE.substring_copy_bynumber(re.match_data, group,
@@ -303,12 +457,15 @@ function _write_capture(io, re, group)
     io.size = max(io.size, io.ptr - 1)
 end
 
-function _replace(io, repl_s::SubstitutionString, str, r, re)
-    SUB_CHAR = '\\'
-    GROUP_CHAR = 'g'
+
+const SUB_CHAR = '\\'
+const GROUP_CHAR = 'g'
+const KEEP_ESC = [SUB_CHAR, GROUP_CHAR, '0':'9'...]
+
+function _replace(io, repl_s::SubstitutionString, str, r, re::RegexAndMatchData)
     LBRACKET = '<'
     RBRACKET = '>'
-    repl = repl_s.string
+    repl = unescape_string(repl_s.string, KEEP_ESC)
     i = firstindex(repl)
     e = lastindex(repl)
     while i <= e
@@ -347,8 +504,8 @@ function _replace(io, repl_s::SubstitutionString, str, r, re)
                 if all(isdigit, groupname)
                     _write_capture(io, re, parse(Int, groupname))
                 else
-                    group = PCRE.substring_number_from_name(re.regex, groupname)
-                    group < 0 && replace_err("Group $groupname not found in regex $re")
+                    group = PCRE.substring_number_from_name(re.re.regex, groupname)
+                    group < 0 && replace_err("Group $groupname not found in regex $(re.re)")
                     _write_capture(io, re, group)
                 end
                 i = nextind(repl, i)
@@ -406,7 +563,7 @@ function iterate(itr::RegexMatchIterator, (offset,prevempty)=(1,false))
 end
 
 """
-    eachmatch(r::Regex, s::AbstractString; overlap::Bool=false])
+    eachmatch(r::Regex, s::AbstractString; overlap::Bool=false)
 
 Search for all matches of a the regular expression `r` in `s` and return a iterator over the
 matches. If overlap is `true`, the matching sequences are allowed to overlap indices in the
@@ -449,3 +606,107 @@ function hash(r::Regex, h::UInt)
     h = hash(r.compile_options, h)
     h = hash(r.match_options, h)
 end
+
+## String operations ##
+
+"""
+    *(s::Regex, t::Union{Regex,AbstractString,AbstractChar}) -> Regex
+    *(s::Union{Regex,AbstractString,AbstractChar}, t::Regex) -> Regex
+
+Concatenate regexes, strings and/or characters, producing a [`Regex`](@ref).
+String and character arguments must be matched exactly in the resulting regex,
+meaning that the contained characters are devoid of any special meaning
+(they are quoted with "\\Q" and "\\E").
+
+!!! compat "Julia 1.3"
+     This method requires at least Julia 1.3.
+
+# Examples
+```jldoctest
+julia> match(r"Hello|Good bye" * ' ' * "world", "Hello world")
+RegexMatch("Hello world")
+
+julia> r = r"a|b" * "c|d"
+r"(?:a|b)\\Qc|d\\E"
+
+julia> match(r, "ac") == nothing
+true
+
+julia> match(r, "ac|d")
+RegexMatch("ac|d")
+```
+"""
+function *(r1::Union{Regex,AbstractString,AbstractChar}, rs::Union{Regex,AbstractString,AbstractChar}...)
+    mask = PCRE.CASELESS | PCRE.MULTILINE | PCRE.DOTALL | PCRE.EXTENDED # imsx
+    match_opts   = nothing # all args must agree on this
+    compile_opts = nothing # all args must agree on this
+    shared = mask
+    for r in (r1, rs...)
+        r isa Regex || continue
+        if match_opts === nothing
+            match_opts = r.match_options
+            compile_opts = r.compile_options & ~mask
+        else
+            r.match_options == match_opts &&
+                r.compile_options & ~mask == compile_opts ||
+                throw(ArgumentError("cannot multiply regexes: incompatible options"))
+        end
+        shared &= r.compile_options
+    end
+    unshared = mask & ~shared
+    Regex(string(wrap_string(r1, unshared), wrap_string.(rs, Ref(unshared))...), compile_opts | shared, match_opts)
+end
+
+*(r::Regex) = r # avoids wrapping r in a useless subpattern
+
+wrap_string(r::Regex, unshared::UInt32) = string("(?", regex_opts_str(r.compile_options & unshared), ':', r.pattern, ')')
+# if s contains raw"\E", split '\' and 'E' within two distinct \Q...\E groups:
+wrap_string(s::AbstractString, ::UInt32) =  string("\\Q", replace(s, raw"\E" => raw"\\E\QE"), "\\E")
+wrap_string(s::AbstractChar, ::UInt32) = string("\\Q", s, "\\E")
+
+regex_opts_str(opts) = (isassigned(_regex_opts_str) ? _regex_opts_str[] : init_regex())[opts]
+
+# UInt32 to String mapping for some compile options
+const _regex_opts_str = Ref{ImmutableDict{UInt32,String}}()
+
+@noinline init_regex() = _regex_opts_str[] = foldl(0:15, init=ImmutableDict{UInt32,String}()) do d, o
+    opt = UInt32(0)
+    str = ""
+    if o & 1 != 0
+        opt |= PCRE.CASELESS
+        str *= 'i'
+    end
+    if o & 2 != 0
+        opt |= PCRE.MULTILINE
+        str *= 'm'
+    end
+    if o & 4 != 0
+        opt |= PCRE.DOTALL
+        str *= 's'
+    end
+    if o & 8 != 0
+        opt |= PCRE.EXTENDED
+        str *= 'x'
+    end
+    ImmutableDict(d, opt => str)
+end
+
+
+"""
+    ^(s::Regex, n::Integer)
+
+Repeat a regex `n` times.
+
+!!! compat "Julia 1.3"
+     This method requires at least Julia 1.3.
+
+# Examples
+```jldoctest
+julia> r"Test "^2
+r"(?:Test ){2}"
+
+julia> match(r"Test "^2, "Test Test ")
+RegexMatch("Test Test ")
+```
+"""
+^(r::Regex, i::Integer) = Regex(string("(?:", r.pattern, "){$i}"), r.compile_options, r.match_options)
