@@ -5,9 +5,12 @@ Experimental multithreading support.
 """
 module Threads
 
+global Condition # we'll define this later, make sure we don't import Base.Condition
+
 include("threadingconstructs.jl")
 include("atomics.jl")
-include("locks.jl")
+include("locks-mt.jl")
+
 
 """
     resize_nthreads!(A, copyvalue=A[1])
