@@ -1,6 +1,7 @@
 // This file is a part of Julia. License is MIT: https://julialang.org/license
 
 #include <limits.h>
+#include <errno.h>
 
 #include "julia.h"
 
@@ -68,6 +69,7 @@ jl_options_t jl_options = { 0,    // quiet
                             NULL, // output-jit-bc
                             NULL, // output-o
                             NULL, // output-ji
+                            NULL,    // output-code_coverage
                             0, // incremental
                             0 // image_file_specified
 };
@@ -75,7 +77,7 @@ jl_options_t jl_options = { 0,    // quiet
 static const char usage[] = "julia [switches] -- [programfile] [args...]\n";
 static const char opts[]  =
     " -v, --version             Display version information\n"
-    " -h, --help                Print this message\n\n"
+    " -h, --help                Print this message (--help-hidden for more)\n\n"
 
     // startup options
     " --project[={<dir>|@.}]    Set <dir> as the home project/environment\n"
@@ -110,7 +112,6 @@ static const char opts[]  =
     " --warn-overwrite={yes|no} Enable or disable method overwrite warnings\n\n"
 
     // code generation options
-    //" --compile={yes|no|all|min}Enable or disable JIT compiler, or request exhaustive compilation\n"
     " -C, --cpu-target <target> Limit usage of CPU features up to <target>; set to \"help\" to see the available options\n"
     " -O, --optimize={0,1,2,3}  Set the optimization level (default level is 2 if unspecified or 3 if used without a level)\n"
     " -g, -g <level>            Enable / Set the level of debug info generation"
@@ -129,19 +130,28 @@ static const char opts[]  =
     // instrumentation options
     " --code-coverage={none|user|all}, --code-coverage\n"
     "                           Count executions of source lines (omitting setting is equivalent to \"user\")\n"
+    " --code-coverage=tracefile.info\n"
+    "                           Append coverage information to the LCOV tracefile (filename supports format tokens).\n"
+// TODO: These TOKENS are defined in `runtime_ccall.cpp`. A more verbose `--help` should include that list here.
     " --track-allocation={none|user|all}, --track-allocation\n"
-    "                           Count bytes allocated by each source line\n\n"
+    "                           Count bytes allocated by each source line (omitting setting is equivalent to \"user\")\n\n"
+;
+
+static const char opts_hidden[]  =
+    // code generation options
+    " --compile={yes|no|all|min}Enable or disable JIT compiler, or request exhaustive compilation\n"
 
     // compiler output options
-    //" --output-o name           Generate an object file (including system image data)\n"
-    //" --output-ji name          Generate a system image data file (.ji)\n"
-// These are for compiler debugging purposes only and should not be otherwise
-// used, so don't show them here. See the devdocs for tips on using these
-// options for debugging the compiler.
-//  " --output-unopt-bc name    Generate unoptimized LLVM bitcode (.bc)\n"
-//  " --output-jit-bc name      Dump all IR generated by the frontend (not including system image)\n"
-    //" --output-bc name          Generate LLVM bitcode (.bc)\n"
-    //" --output-incremental=no   Generate an incremental output file (rather than complete)\n\n"
+    " --output-o name           Generate an object file (including system image data)\n"
+    " --output-ji name          Generate a system image data file (.ji)\n"
+
+    // compiler debugging (see the devdocs for tips on using these options)
+    " --output-unopt-bc name    Generate unoptimized LLVM bitcode (.bc)\n"
+    " --output-jit-bc name      Dump all IR generated by the frontend (not including system image)\n"
+    " --output-bc name          Generate LLVM bitcode (.bc)\n"
+    " --output-incremental=no   Generate an incremental output file (rather than complete)\n"
+    " --trace-compile={stdout,stderr}\n"
+    "                           Print precompile statements for methods compiled during execution.\n\n"
 ;
 
 JL_DLLEXPORT void jl_parse_opts(int *argcp, char ***argvp)
@@ -171,6 +181,7 @@ JL_DLLEXPORT void jl_parse_opts(int *argcp, char ***argvp)
            opt_use_precompiled,
            opt_use_compilecache,
            opt_incremental,
+           opt_help_hidden,
            opt_banner,
            opt_sysimage_native_code,
            opt_compiled_modules,
@@ -184,6 +195,7 @@ JL_DLLEXPORT void jl_parse_opts(int *argcp, char ***argvp)
         // with the required arguments defined in base/client.jl `process_options()`
         { "version",         no_argument,       0, 'v' },
         { "help",            no_argument,       0, 'h' },
+        { "help-hidden",     no_argument,       0, opt_help_hidden },
         { "quiet",           no_argument,       0, 'q' },
         { "banner",          required_argument, 0, opt_banner },
         { "home",            required_argument, 0, 'H' },
@@ -287,10 +299,13 @@ restart_switch:
             break;
         case 'v': // version
             jl_printf(JL_STDOUT, "julia version %s\n", JULIA_VERSION_STRING);
-            jl_exit(0);
+            exit(0);
         case 'h': // help
             jl_printf(JL_STDOUT, "%s%s", usage, opts);
-            jl_exit(0);
+            exit(0);
+        case opt_help_hidden:
+            jl_printf(JL_STDOUT, "%s%s", usage, opts_hidden);
+            exit(0);
         case 'g': // debug info
             if (optarg != NULL) {
                 if (!strcmp(optarg,"0"))
@@ -317,17 +332,11 @@ restart_switch:
         case 'L': // load
         {
             size_t sz = strlen(optarg) + 1;
-            char *arg = (char*)malloc(sz + 1);
+            char *arg = (char*)malloc_s(sz + 1);
             const char **newcmds;
-            if (!arg)
-                jl_errorf("fatal error: failed to allocate memory: %s", strerror(errno));
             arg[0] = c;
             memcpy(arg + 1, optarg, sz);
-            newcmds = (const char**)realloc(cmds, (ncmds + 2) * sizeof(char*));
-            if (!newcmds) {
-                free(cmds);
-                jl_errorf("fatal error: failed to allocate memory: %s", strerror(errno));
-            }
+            newcmds = (const char**)realloc_s(cmds, (ncmds + 2) * sizeof(char*));
             cmds = newcmds;
             cmds[ncmds] = arg;
             ncmds++;
@@ -437,12 +446,18 @@ restart_switch:
             break;
         case opt_code_coverage:
             if (optarg != NULL) {
-                if (!strcmp(optarg,"user"))
+                size_t endof = strlen(optarg);
+                if (!strcmp(optarg, "user"))
                     codecov = JL_LOG_USER;
-                else if (!strcmp(optarg,"all"))
+                else if (!strcmp(optarg, "all"))
                     codecov = JL_LOG_ALL;
-                else if (!strcmp(optarg,"none"))
+                else if (!strcmp(optarg, "none"))
                     codecov = JL_LOG_NONE;
+                else if (endof > 5 && !strcmp(optarg + endof - 5, ".info")) {
+                    if (codecov == JL_LOG_NONE)
+                        codecov = JL_LOG_ALL;
+                    jl_options.output_code_coverage = optarg;
+                }
                 else
                     jl_errorf("julia: invalid argument to --code-coverage (%s)", optarg);
                 break;
@@ -602,28 +617,9 @@ restart_switch:
     }
     jl_options.code_coverage = codecov;
     jl_options.malloc_log = malloclog;
-    *argvp += optind;
-    *argcp -= optind;
-}
-
-JL_DLLEXPORT void jl_set_ARGS(int argc, char **argv)
-{
-    if (jl_core_module != NULL) {
-        jl_array_t *args = (jl_array_t*)jl_get_global(jl_core_module, jl_symbol("ARGS"));
-        if (args == NULL) {
-            args = jl_alloc_vec_any(0);
-            JL_GC_PUSH1(&args);
-            jl_set_const(jl_core_module, jl_symbol("ARGS"), (jl_value_t*)args);
-            JL_GC_POP();
-        }
-        assert(jl_array_len(args) == 0);
-        jl_array_grow_end(args, argc);
-        int i;
-        for (i=0; i < argc; i++) {
-            jl_value_t *s = (jl_value_t*)jl_cstr_to_string(argv[i]);
-            jl_arrayset(args, s, i);
-        }
-    }
+    int proc_args = *argcp < optind ? *argcp : optind;
+    *argvp += proc_args;
+    *argcp -= proc_args;
 }
 
 JL_DLLEXPORT ssize_t jl_sizeof_jl_options(void)
