@@ -9,8 +9,12 @@
 #include "llvm-version.h"
 #include "support/dtypes.h"
 
+#include <llvm-c/Core.h>
+#include <llvm-c/Types.h>
+
 #include <llvm/Pass.h>
 #include <llvm/IR/Module.h>
+#include <llvm/IR/LegacyPassManager.h>
 #include <llvm/IR/Function.h>
 #include <llvm/IR/Instructions.h>
 #include <llvm/IR/Constants.h>
@@ -18,7 +22,6 @@
 #include <llvm/Analysis/LoopInfo.h>
 #include <llvm/Analysis/CallGraph.h>
 #include <llvm/IR/LegacyPassManager.h>
-#include <llvm/IR/MDBuilder.h>
 #include <llvm/IR/IRBuilder.h>
 #include <llvm/IR/DebugInfoMetadata.h>
 #include <llvm/Transforms/Utils/Cloning.h>
@@ -1078,3 +1081,8 @@ Pass *createMultiVersioningPass()
 {
     return new MultiVersioning();
 }
+
+extern "C" JL_DLLEXPORT void LLVMExtraAddMultiVersioningPass(LLVMPassManagerRef PM)
+{
+    unwrap(PM)->add(createMultiVersioningPass());
+}
