@@ -43,8 +43,8 @@ JL_DLLEXPORT void JL_NORETURN jl_error(const char *str)
 
 extern int vasprintf(char **str, const char *fmt, va_list ap);
 
-static jl_value_t *jl_vexceptionf(jl_datatype_t *exception_type,
-                                  const char *fmt, va_list args)
+jl_value_t *jl_vexceptionf(jl_datatype_t *exception_type,
+                           const char *fmt, va_list args)
 {
     if (exception_type == NULL) {
         jl_printf(JL_STDERR, "ERROR: ");
@@ -107,28 +107,24 @@ JL_DLLEXPORT void JL_NORETURN jl_too_many_args(const char *fname, int max)
     jl_exceptionf(jl_argumenterror_type, "%s: too many arguments (expected %d)", fname, max);
 }
 
+// with function name / location description, plus extra context
 JL_DLLEXPORT void JL_NORETURN jl_type_error_rt(const char *fname, const char *context,
-                                               jl_value_t *ty, jl_value_t *got)
+                                               jl_value_t *expected, jl_value_t *got)
 {
     jl_value_t *ctxt=NULL;
-    JL_GC_PUSH2(&ctxt, &got);
+    JL_GC_PUSH3(&ctxt, &expected, &got);
     ctxt = jl_pchar_to_string((char*)context, strlen(context));
-    jl_value_t *ex = jl_new_struct(jl_typeerror_type, jl_symbol(fname),
-                                   ctxt, ty, got);
+    jl_value_t *ex = jl_new_struct(jl_typeerror_type, jl_symbol(fname), ctxt, expected, got);
     jl_throw(ex);
 }
 
+// with function name or description only
 JL_DLLEXPORT void JL_NORETURN jl_type_error(const char *fname, jl_value_t *expected,
                                             jl_value_t *got)
 {
     jl_type_error_rt(fname, "", expected, got);
 }
 
-JL_DLLEXPORT void JL_NORETURN jl_type_error_new_expr(jl_value_t *ty, jl_value_t *got)
-{
-    jl_type_error_rt("Type", "new", ty, got);
-}
-
 JL_DLLEXPORT void JL_NORETURN jl_undefined_var_error(jl_sym_t *var)
 {
     jl_throw(jl_new_struct(jl_undefvarerror_type, var));
@@ -192,16 +188,13 @@ JL_DLLEXPORT void JL_NORETURN jl_eof_error(void)
     jl_datatype_t *eof_error =
         (jl_datatype_t*)jl_get_global(jl_base_module, jl_symbol("EOFError"));
     assert(eof_error != NULL);
-    jl_exceptionf(eof_error, "");
+    jl_throw(jl_new_struct(eof_error));
 }
 
 // get kwsorter field, with appropriate error check and message
 JL_DLLEXPORT jl_value_t *jl_get_keyword_sorter(jl_value_t *f)
 {
-    jl_methtable_t *mt = jl_gf_mtable(f);
-    if (mt->kwsorter == NULL)
-        jl_errorf("function %s does not accept keyword arguments", jl_symbol_name(mt->name));
-    return mt->kwsorter;
+    return jl_get_kwsorter(jl_typeof(f));
 }
 
 JL_DLLEXPORT void jl_typeassert(jl_value_t *x, jl_value_t *t)
@@ -210,6 +203,8 @@ JL_DLLEXPORT void jl_typeassert(jl_value_t *x, jl_value_t *t)
         jl_type_error("typeassert", t, x);
 }
 
+// exceptions -----------------------------------------------------------------
+
 JL_DLLEXPORT void jl_enter_handler(jl_handler_t *eh)
 {
     jl_ptls_t ptls = jl_get_ptls_states();
@@ -217,10 +212,8 @@ JL_DLLEXPORT void jl_enter_handler(jl_handler_t *eh)
     // Must have no safepoint
     eh->prev = current_task->eh;
     eh->gcstack = ptls->pgcstack;
-#ifdef JULIA_ENABLE_THREADING
     eh->gc_state = ptls->gc_state;
     eh->locks_len = current_task->locks.len;
-#endif
     eh->defer_signal = ptls->defer_signal;
     eh->finalizers_inhibited = ptls->finalizers_inhibited;
     eh->world_age = ptls->world_age;
@@ -230,6 +223,46 @@ JL_DLLEXPORT void jl_enter_handler(jl_handler_t *eh)
 #endif
 }
 
+// Restore thread local state to saved state in error handler `eh`.
+// This is executed in two circumstances:
+// * We leave a try block through normal control flow
+// * An exception causes a nonlocal jump to the catch block. In this case
+//   there's additional cleanup required, eg pushing the exception stack.
+JL_DLLEXPORT void jl_eh_restore_state(jl_handler_t *eh)
+{
+    jl_ptls_t ptls = jl_get_ptls_states();
+#ifdef _OS_WINDOWS_
+    if (ptls->needs_resetstkoflw) {
+        _resetstkoflw();
+        ptls->needs_resetstkoflw = 0;
+    }
+#endif
+    jl_task_t *current_task = ptls->current_task;
+    // `eh` may be not equal to `ptls->current_task->eh`. See `jl_pop_handler`
+    // This function should **NOT** have any safepoint before the ones at the
+    // end.
+    sig_atomic_t old_defer_signal = ptls->defer_signal;
+    int8_t old_gc_state = ptls->gc_state;
+    current_task->eh = eh->prev;
+    ptls->pgcstack = eh->gcstack;
+    arraylist_t *locks = &current_task->locks;
+    if (locks->len > eh->locks_len) {
+        for (size_t i = locks->len;i > eh->locks_len;i--)
+            jl_mutex_unlock_nogc((jl_mutex_t*)locks->items[i - 1]);
+        locks->len = eh->locks_len;
+    }
+    ptls->world_age = eh->world_age;
+    ptls->defer_signal = eh->defer_signal;
+    ptls->gc_state = eh->gc_state;
+    ptls->finalizers_inhibited = eh->finalizers_inhibited;
+    if (old_gc_state && !eh->gc_state) {
+        jl_gc_safepoint_(ptls);
+    }
+    if (old_defer_signal && !eh->defer_signal) {
+        jl_sigint_safepoint(ptls);
+    }
+}
+
 JL_DLLEXPORT void jl_pop_handler(int n)
 {
     jl_ptls_t ptls = jl_get_ptls_states();
@@ -241,61 +274,56 @@ JL_DLLEXPORT void jl_pop_handler(int n)
     jl_eh_restore_state(eh);
 }
 
-JL_DLLEXPORT jl_value_t *jl_apply_with_saved_exception_state(jl_value_t **args, uint32_t nargs, int drop_exceptions)
+JL_DLLEXPORT size_t jl_excstack_state(void)
 {
     jl_ptls_t ptls = jl_get_ptls_states();
-    jl_value_t *exc = ptls->exception_in_transit;
-    jl_array_t *bt = NULL;
-    jl_array_t *bt2 = NULL;
-    JL_GC_PUSH3(&exc, &bt, &bt2);
-    if (ptls->bt_size > 0) {
-        jl_get_backtrace(&bt, &bt2);
-        ptls->bt_size = 0;
-    }
-    jl_value_t *v;
-    JL_TRY {
-        v = jl_apply(args, nargs);
-    }
-    JL_CATCH {
-        if (!drop_exceptions) {
-            jl_printf(JL_STDERR, "Internal error: encountered unexpected error in runtime:\n");
-            jl_static_show(JL_STDERR, ptls->exception_in_transit);
-            jl_printf(JL_STDERR, "\n");
-            jlbacktrace(); // written to STDERR_FILENO
-        }
-        v = NULL;
-    }
-    ptls->exception_in_transit = exc;
-    if (bt != NULL) {
-        // This is sufficient because bt2 roots the gc-managed values
-        memcpy(ptls->bt_data, bt->data, jl_array_len(bt) * sizeof(void*));
-        ptls->bt_size = jl_array_len(bt);
-    }
-    JL_GC_POP();
-    return v;
+    jl_excstack_t *s = ptls->current_task->excstack;
+    return s ? s->top : 0;
 }
 
-// misc -----------------------------------------------------------------------
-
-// perform f(args...) on stack
-JL_DLLEXPORT jl_value_t *jl_apply_2va(jl_value_t *f, jl_value_t **args, uint32_t nargs)
+JL_DLLEXPORT void jl_restore_excstack(size_t state)
 {
-    nargs++;
-    int onstack = (nargs < jl_page_size/sizeof(jl_value_t*));
-    jl_value_t **newargs;
-    JL_GC_PUSHARGS(newargs, onstack ? nargs : 1);
-    jl_svec_t *arg_heap = NULL;
-    newargs[0] = f;  // make sure f is rooted
-    if (!onstack) {
-        arg_heap = jl_alloc_svec(nargs);
-        newargs[0] = (jl_value_t*)arg_heap;
-        newargs = jl_svec_data(arg_heap);
-        newargs[0] = f;
+    jl_ptls_t ptls = jl_get_ptls_states();
+    jl_excstack_t *s = ptls->current_task->excstack;
+    if (s) {
+        assert(s->top >= state);
+        s->top = state;
     }
-    memcpy(&newargs[1], args, (nargs-1)*sizeof(jl_value_t*));
-    jl_value_t *ret = jl_apply_generic(newargs, nargs);
-    JL_GC_POP();
-    return ret;
+}
+
+void jl_copy_excstack(jl_excstack_t *dest, jl_excstack_t *src) JL_NOTSAFEPOINT
+{
+    assert(dest->reserved_size >= src->top);
+    memcpy(jl_excstack_raw(dest), jl_excstack_raw(src), sizeof(jl_bt_element_t)*src->top);
+    dest->top = src->top;
+}
+
+void jl_reserve_excstack(jl_excstack_t **stack JL_REQUIRE_ROOTED_SLOT,
+                          size_t reserved_size)
+{
+    jl_excstack_t *s = *stack;
+    if (s && s->reserved_size >= reserved_size)
+        return;
+    size_t bufsz = sizeof(jl_excstack_t) + sizeof(uintptr_t)*reserved_size;
+    jl_excstack_t *new_s = (jl_excstack_t*)jl_gc_alloc_buf(jl_get_ptls_states(), bufsz);
+    new_s->top = 0;
+    new_s->reserved_size = reserved_size;
+    if (s)
+        jl_copy_excstack(new_s, s);
+    *stack = new_s;
+}
+
+void jl_push_excstack(jl_excstack_t **stack JL_REQUIRE_ROOTED_SLOT JL_ROOTING_ARGUMENT,
+                      jl_value_t *exception JL_ROOTED_ARGUMENT,
+                      jl_bt_element_t *bt_data, size_t bt_size)
+{
+    jl_reserve_excstack(stack, (*stack ? (*stack)->top : 0) + bt_size + 2);
+    jl_excstack_t *s = *stack;
+    jl_bt_element_t *rawstack = jl_excstack_raw(s);
+    memcpy(rawstack + s->top, bt_data, sizeof(jl_bt_element_t)*bt_size);
+    s->top += bt_size + 2;
+    rawstack[s->top-2].uintptr = bt_size;
+    rawstack[s->top-1].jlvalue = exception;
 }
 
 // conversion -----------------------------------------------------------------
@@ -314,11 +342,23 @@ JL_DLLEXPORT jl_value_t *jl_value_ptr(jl_value_t *a)
 {
     return a;
 }
-JL_DLLEXPORT void jl_gc_use(jl_value_t *a)
+
+// optimization of setfield which bypasses boxing of the idx (and checking field type validity)
+JL_DLLEXPORT void jl_set_nth_field(jl_value_t *v, size_t idx0, jl_value_t *rhs)
 {
-    (void)a;
+    jl_datatype_t *st = (jl_datatype_t*)jl_typeof(v);
+    if (!st->mutabl)
+        jl_errorf("setfield! immutable struct of type %s cannot be changed", jl_symbol_name(st->name->name));
+    if (idx0 >= jl_datatype_nfields(st))
+        jl_bounds_error_int(v, idx0 + 1);
+    //jl_value_t *ft = jl_field_type(st, idx0);
+    //if (!jl_isa(rhs, ft)) {
+    //    jl_type_error("setfield!", ft, rhs);
+    //}
+    set_nth_field(st, (void*)v, idx0, rhs);
 }
 
+
 // parsing --------------------------------------------------------------------
 
 int substr_isspace(char *p, char *pend)
@@ -359,7 +399,7 @@ JL_DLLEXPORT jl_nullable_float64_t jl_try_substrtod(char *str, size_t offset, si
             newstr = (char*)alloca(len + 1);
         }
         else {
-            newstr = tofree = (char*)malloc(len + 1);
+            newstr = tofree = (char*)malloc_s(len + 1);
         }
         memcpy(newstr, bstr, len);
         newstr[len] = 0;
@@ -418,7 +458,7 @@ JL_DLLEXPORT jl_nullable_float32_t jl_try_substrtof(char *str, size_t offset, si
             newstr = (char*)alloca(len + 1);
         }
         else {
-            newstr = tofree = (char*)malloc(len + 1);
+            newstr = tofree = (char*)malloc_s(len + 1);
         }
         memcpy(newstr, bstr, len);
         newstr[len] = 0;
@@ -462,29 +502,31 @@ JL_DLLEXPORT int jl_substrtof(char *str, int offset, size_t len, float *out)
 
 // showing --------------------------------------------------------------------
 
-JL_DLLEXPORT void jl_flush_cstdio(void)
+JL_DLLEXPORT void jl_flush_cstdio(void) JL_NOTSAFEPOINT
 {
     fflush(stdout);
     fflush(stderr);
 }
 
-JL_DLLEXPORT jl_value_t *jl_stdout_obj(void)
+JL_DLLEXPORT jl_value_t *jl_stdout_obj(void) JL_NOTSAFEPOINT
 {
-    if (jl_base_module == NULL) return NULL;
-    jl_value_t *stdout_obj = jl_get_global(jl_base_module, jl_symbol("stdout"));
-    return stdout_obj;
+    if (jl_base_module == NULL)
+        return NULL;
+    jl_binding_t *stdout_obj = jl_get_module_binding(jl_base_module, jl_symbol("stdout"));
+    return stdout_obj ? stdout_obj->value : NULL;
 }
 
-JL_DLLEXPORT jl_value_t *jl_stderr_obj(void)
+JL_DLLEXPORT jl_value_t *jl_stderr_obj(void) JL_NOTSAFEPOINT
 {
-    if (jl_base_module == NULL) return NULL;
-    jl_value_t *stderr_obj = jl_get_global(jl_base_module, jl_symbol("stderr"));
-    return stderr_obj;
+    if (jl_base_module == NULL)
+        return NULL;
+    jl_binding_t *stderr_obj = jl_get_module_binding(jl_base_module, jl_symbol("stderr"));
+    return stderr_obj ? stderr_obj->value : NULL;
 }
 
 // toys for debugging ---------------------------------------------------------
 
-static size_t jl_show_svec(JL_STREAM *out, jl_svec_t *t, const char *head, const char *opn, const char *cls)
+static size_t jl_show_svec(JL_STREAM *out, jl_svec_t *t, const char *head, const char *opn, const char *cls) JL_NOTSAFEPOINT
 {
     size_t i, n=0, len = jl_svec_len(t);
     n += jl_printf(out, "%s", head);
@@ -504,12 +546,13 @@ struct recur_list {
     jl_value_t *v;
 };
 
-static size_t jl_static_show_x(JL_STREAM *out, jl_value_t *v, struct recur_list *depth);
+static size_t jl_static_show_x(JL_STREAM *out, jl_value_t *v, struct recur_list *depth) JL_NOTSAFEPOINT;
+static size_t jl_static_show_next_(JL_STREAM *out, jl_value_t *v, jl_value_t *prev, struct recur_list *depth) JL_NOTSAFEPOINT;
 
-JL_DLLEXPORT int jl_id_start_char(uint32_t wc);
-JL_DLLEXPORT int jl_id_char(uint32_t wc);
+JL_DLLEXPORT int jl_id_start_char(uint32_t wc) JL_NOTSAFEPOINT;
+JL_DLLEXPORT int jl_id_char(uint32_t wc) JL_NOTSAFEPOINT;
 
-JL_DLLEXPORT int jl_is_identifier(char *str)
+JL_DLLEXPORT int jl_is_identifier(char *str) JL_NOTSAFEPOINT
 {
     size_t i = 0;
     uint32_t wc = u8_nextchar(str, &i);
@@ -522,13 +565,58 @@ JL_DLLEXPORT int jl_is_identifier(char *str)
     return 1;
 }
 
+static jl_datatype_t *first_arg_datatype(jl_value_t *a JL_PROPAGATES_ROOT, int got_tuple1) JL_NOTSAFEPOINT
+{
+    if (jl_is_datatype(a)) {
+        if (got_tuple1)
+            return (jl_datatype_t*)a;
+        if (jl_is_tuple_type(a)) {
+            if (jl_nparams(a) < 1)
+                return NULL;
+            return first_arg_datatype(jl_tparam0(a), 1);
+        }
+        return NULL;
+    }
+    else if (jl_is_typevar(a)) {
+        return first_arg_datatype(((jl_tvar_t*)a)->ub, got_tuple1);
+    }
+    else if (jl_is_unionall(a)) {
+        return first_arg_datatype(((jl_unionall_t*)a)->body, got_tuple1);
+    }
+    else if (jl_is_uniontype(a)) {
+        jl_uniontype_t *u = (jl_uniontype_t*)a;
+        jl_datatype_t *d1 = first_arg_datatype(u->a, got_tuple1);
+        if (d1 == NULL) return NULL;
+        jl_datatype_t *d2 = first_arg_datatype(u->b, got_tuple1);
+        if (d2 == NULL || d1->name != d2->name)
+            return NULL;
+        return d1;
+    }
+    return NULL;
+}
+
+// get DataType of first tuple element (if present), or NULL if cannot be determined
+JL_DLLEXPORT jl_datatype_t *jl_first_argument_datatype(jl_value_t *argtypes JL_PROPAGATES_ROOT) JL_NOTSAFEPOINT
+{
+    return first_arg_datatype(argtypes, 0);
+}
+
+// get DataType implied by a single given type, or `nothing`
+JL_DLLEXPORT jl_value_t *jl_argument_datatype(jl_value_t *argt JL_PROPAGATES_ROOT) JL_NOTSAFEPOINT
+{
+    jl_datatype_t *dt = first_arg_datatype(argt, 1);
+    if (dt == NULL)
+        return jl_nothing;
+    return (jl_value_t*)dt;
+}
+
 // `v` might be pointing to a field inlined in a structure therefore
 // `jl_typeof(v)` may not be the same with `vt` and only `vt` should be
 // used to determine the type of the value.
 // This is necessary to make sure that this function doesn't allocate any
 // memory through the Julia GC
 static size_t jl_static_show_x_(JL_STREAM *out, jl_value_t *v, jl_datatype_t *vt,
-                                struct recur_list *depth)
+                                struct recur_list *depth) JL_NOTSAFEPOINT
 {
     size_t n = 0;
     if ((uintptr_t)vt < 4096U) {
@@ -539,6 +627,26 @@ static size_t jl_static_show_x_(JL_STREAM *out, jl_value_t *v, jl_datatype_t *vt
         n += jl_static_show_x(out, (jl_value_t*)vt, depth);
         n += jl_printf(out, ">");
     }
+    // These need to be special cased because they
+    // exist only by pointer identity in early startup
+    else if (v == (jl_value_t*)jl_simplevector_type) {
+        n += jl_printf(out, "Core.SimpleVector");
+    }
+    else if (v == (jl_value_t*)jl_typename_type) {
+        n += jl_printf(out, "Core.TypeName");
+    }
+    else if (v == (jl_value_t*)jl_symbol_type) {
+        n += jl_printf(out, "Symbol");
+    }
+    else if (v == (jl_value_t*)jl_methtable_type) {
+        n += jl_printf(out, "Core.MethodTable");
+    }
+    else if (v == (jl_value_t*)jl_any_type) {
+        n += jl_printf(out, "Any");
+    }
+    else if (v == (jl_value_t*)jl_type_type) {
+        n += jl_printf(out, "Type");
+    }
     else if (vt == jl_method_type) {
         jl_method_t *m = (jl_method_t*)v;
         n += jl_static_show_x(out, (jl_value_t*)m->module, depth);
@@ -559,7 +667,7 @@ static size_t jl_static_show_x_(JL_STREAM *out, jl_value_t *v, jl_datatype_t *vt
         else {
             n += jl_static_show_x(out, (jl_value_t*)li->def.module, depth);
             n += jl_printf(out, ".<toplevel thunk> -> ");
-            n += jl_static_show_x(out, li->inferred, depth);
+            n += jl_static_show_x(out, li->uninferred, depth);
         }
     }
     else if (vt == jl_simplevector_type) {
@@ -576,8 +684,8 @@ static size_t jl_static_show_x_(JL_STREAM *out, jl_value_t *v, jl_datatype_t *vt
         if (globname && !strchr(jl_symbol_name(globname), '#') &&
             !strchr(jl_symbol_name(globname), '@') && dv->name->module &&
             jl_binding_resolved_p(dv->name->module, globname)) {
-            jl_binding_t *b = jl_get_binding(dv->name->module, globname);
-            if (b && jl_typeof(b->value) == v)
+            jl_binding_t *b = jl_get_module_binding(dv->name->module, globname);
+            if (b && b->value && jl_typeof(b->value) == v)
                 globfunc = 1;
         }
         jl_sym_t *sym = globfunc ? globname : dv->name->name;
@@ -591,7 +699,7 @@ static size_t jl_static_show_x_(JL_STREAM *out, jl_value_t *v, jl_datatype_t *vt
         else if (globfunc) {
             n += jl_printf(out, "typeof(");
         }
-        if (dv->name->module != jl_core_module || !jl_module_exports_p(jl_core_module, sym)) {
+        if (jl_core_module && (dv->name->module != jl_core_module || !jl_module_exports_p(jl_core_module, sym))) {
             n += jl_static_show_x(out, (jl_value_t*)dv->name->module, depth);
             if (!hidden) {
                 n += jl_printf(out, ".");
@@ -661,7 +769,7 @@ static size_t jl_static_show_x_(JL_STREAM *out, jl_value_t *v, jl_datatype_t *vt
     else if (vt == jl_uint8_type) {
         n += jl_printf(out, "0x%02" PRIx8, *(uint8_t*)v);
     }
-    else if (jl_is_cpointer_type((jl_value_t*)vt)) {
+    else if (jl_pointer_type && jl_is_cpointer_type((jl_value_t*)vt)) {
 #ifdef _P64
         n += jl_printf(out, "0x%016" PRIx64, *(uint64_t*)v);
 #else
@@ -677,7 +785,7 @@ static size_t jl_static_show_x_(JL_STREAM *out, jl_value_t *v, jl_datatype_t *vt
     else if (vt == jl_bool_type) {
         n += jl_printf(out, "%s", *(uint8_t*)v ? "true" : "false");
     }
-    else if ((jl_value_t*)vt == jl_typeof(jl_nothing)) {
+    else if (v == jl_nothing || (jl_nothing && (jl_value_t*)vt == jl_typeof(jl_nothing))) {
         n += jl_printf(out, "nothing");
     }
     else if (vt == jl_string_type) {
@@ -747,7 +855,7 @@ static size_t jl_static_show_x_(JL_STREAM *out, jl_value_t *v, jl_datatype_t *vt
         }
         n += jl_printf(out, "%s", jl_symbol_name(m->name));
     }
-    else if (vt == jl_sym_type) {
+    else if (vt == jl_symbol_type) {
         char *sn = jl_symbol_name((jl_sym_t*)v);
         int quoted = !jl_is_identifier(sn) && jl_operator_precedence(sn) == 0;
         if (quoted)
@@ -814,7 +922,7 @@ static size_t jl_static_show_x_(JL_STREAM *out, jl_value_t *v, jl_datatype_t *vt
             n += jl_printf(out, ")");
         }
     }
-    else if (jl_is_array_type(vt)) {
+    else if (jl_array_type && jl_is_array_type(vt)) {
         n += jl_printf(out, "Array{");
         n += jl_static_show_x(out, (jl_value_t*)jl_tparam0(vt), depth);
         n += jl_printf(out, ", (");
@@ -873,7 +981,7 @@ static size_t jl_static_show_x_(JL_STREAM *out, jl_value_t *v, jl_datatype_t *vt
         n += jl_static_show_x(out, *(jl_value_t**)v, depth);
         n += jl_printf(out, ")");
     }
-    else if (jl_is_datatype(vt)) {
+    else if (jl_datatype_type && jl_is_datatype(vt)) {
         int istuple = jl_is_tuple_type(vt), isnamedtuple = jl_is_namedtuple_type(vt);
         size_t tlen = jl_datatype_nfields(vt);
         if (isnamedtuple) {
@@ -906,7 +1014,7 @@ static size_t jl_static_show_x_(JL_STREAM *out, jl_value_t *v, jl_datatype_t *vt
                     n += jl_static_show_x(out, *(jl_value_t**)fld_ptr, depth);
                 }
                 else {
-                    jl_datatype_t *ft = (jl_datatype_t*)jl_field_type(vt, i);
+                    jl_datatype_t *ft = (jl_datatype_t*)jl_field_type_concrete(vt, i);
                     if (jl_is_uniontype(ft)) {
                         uint8_t sel = ((uint8_t*)fld_ptr)[jl_field_size(vt, i) - 1];
                         ft = (jl_datatype_t*)jl_nth_union_component((jl_value_t*)ft, sel);
@@ -920,7 +1028,7 @@ static size_t jl_static_show_x_(JL_STREAM *out, jl_value_t *v, jl_datatype_t *vt
             }
             if (vt == jl_typemap_entry_type) {
                 n += jl_printf(out, ", next=↩︎\n  ");
-                n += jl_static_show_x(out, jl_fieldref_noalloc(v, 0), depth);
+                n += jl_static_show_next_(out, (jl_value_t*)((jl_typemap_entry_t*)v)->next, v, depth);
             }
         }
         n += jl_printf(out, ")");
@@ -933,9 +1041,16 @@ static size_t jl_static_show_x_(JL_STREAM *out, jl_value_t *v, jl_datatype_t *vt
     return n;
 }
 
-static size_t jl_static_show_x(JL_STREAM *out, jl_value_t *v, struct recur_list *depth)
+static size_t jl_static_show_x(JL_STREAM *out, jl_value_t *v, struct recur_list *depth) JL_NOTSAFEPOINT
 {
     // show values without calling a julia method or allocating through the GC
+    return jl_static_show_next_(out, v, NULL, depth);
+}
+
+static size_t jl_static_show_next_(JL_STREAM *out, jl_value_t *v, jl_value_t *prev, struct recur_list *depth) JL_NOTSAFEPOINT
+{
+    // helper for showing a typemap list by following the next pointers
+    // while being careful about avoiding any recursion due to malformed (circular) references
     if (v == NULL) {
         return jl_printf(out, "#<null>");
     }
@@ -943,22 +1058,56 @@ static size_t jl_static_show_x(JL_STREAM *out, jl_value_t *v, struct recur_list
         return jl_printf(out, "#<%d>", (int)(uintptr_t)v);
     }
     unsigned int dist = 1;
-    struct recur_list this_item = {depth, v}, *p = depth;
+    struct recur_list this_item = {depth, v},
+                      *newdepth = &this_item,
+                      *p = depth;
     while (p) {
+        if (jl_typeis(v, jl_typemap_entry_type) && newdepth == &this_item) {
+            jl_value_t *m = p->v;
+            unsigned nid = 1;
+            while (m && jl_typeis(m, jl_typemap_entry_type)) {
+                if (m == v) {
+                    return jl_printf(out, "<typemap reference #%u @-%u ", nid, dist) +
+                           jl_static_show_x(out, (jl_value_t*)((jl_typemap_entry_t*)m)->sig, depth) +
+                           jl_printf(out, ">");
+                }
+                if (m == prev) {
+                    newdepth = depth;
+                    break;
+                }
+                // verify that we aren't trying to follow a circular list
+                // by following the list again, and ensuring this is the only link to next
+                jl_value_t *mnext = (jl_value_t*)((jl_typemap_entry_t*)m)->next;
+                jl_value_t *m2 = p->v;
+                if (m2 == mnext)
+                    break;
+                while (m2 && jl_typeis(m2, jl_typemap_entry_type)) {
+                    jl_value_t *mnext2 = (jl_value_t*)((jl_typemap_entry_t*)m2)->next;
+                    if (mnext2 == mnext) {
+                        if (m2 != m)
+                            mnext = NULL;
+                        break;
+                    }
+                    m2 = mnext2;
+                }
+                m = mnext;
+                nid++;
+            }
+        }
         if (p->v == v)
             return jl_printf(out, "<circular reference @-%u>", dist);
         dist++;
         p = p->prev;
     }
-    return jl_static_show_x_(out, v, (jl_datatype_t*)jl_typeof(v), &this_item);
+    return jl_static_show_x_(out, v, (jl_datatype_t*)jl_typeof(v), newdepth);
 }
 
-JL_DLLEXPORT size_t jl_static_show(JL_STREAM *out, jl_value_t *v)
+JL_DLLEXPORT size_t jl_static_show(JL_STREAM *out, jl_value_t *v) JL_NOTSAFEPOINT
 {
     return jl_static_show_x(out, v, 0);
 }
 
-JL_DLLEXPORT size_t jl_static_show_func_sig(JL_STREAM *s, jl_value_t *type)
+JL_DLLEXPORT size_t jl_static_show_func_sig(JL_STREAM *s, jl_value_t *type) JL_NOTSAFEPOINT
 {
     jl_value_t *ftype = (jl_value_t*)jl_first_argument_datatype(type);
     if (ftype == NULL)
@@ -1015,7 +1164,7 @@ JL_DLLEXPORT size_t jl_static_show_func_sig(JL_STREAM *s, jl_value_t *type)
     return n;
 }
 
-JL_DLLEXPORT void jl_(void *jl_value)
+JL_DLLEXPORT void jl_(void *jl_value) JL_NOTSAFEPOINT
 {
     jl_ptls_t ptls = jl_get_ptls_states();
     jl_jmp_buf *old_buf = ptls->safe_restore;
@@ -1052,7 +1201,7 @@ void jl_log(int level, jl_value_t *module, jl_value_t *group, jl_value_t *id,
     if (!logmsg_func) {
         ios_t str_;
         ios_mem(&str_, 300);
-        uv_stream_t* str = (uv_stream_t*)&str_;
+        JL_STREAM* str = (JL_STREAM*)&str_;
         if (jl_is_string(msg)) {
             jl_uv_puts(str, jl_string_data(msg), jl_string_len(msg));
         }
