@@ -2,8 +2,8 @@
 // MurmurHash3 was written by Austin Appleby, and is placed in the public
 // domain. The author hereby disclaims copyright to this source code.
 
-#ifndef MURMURHASH3_H
-#define MURMURHASH3_H
+#ifndef JL_MURMURHASH3_H
+#define JL_MURMURHASH3_H
 
 //-----------------------------------------------------------------------------
 // Platform-specific functions and macros
