@@ -1,5 +1,13 @@
 // This file is a part of Julia. License is MIT: https://julialang.org/license
 
+#ifndef __has_feature
+#define __has_feature(x) 0
+#endif
+#if !(defined(__clang__) && __has_feature(nullability))
+#define _Nonnull
+#endif
+#define JL_NONNULL _Nonnull
+
 #ifdef __clang_analyzer__
 
 #define JL_PROPAGATES_ROOT __attribute__((annotate("julia_propagates_root")))
@@ -9,14 +17,15 @@
 #define JL_ROOTING_ARGUMENT __attribute__((annotate("julia_rooting_argument")))
 #define JL_ROOTED_ARGUMENT __attribute__((annotate("julia_rooted_argument")))
 #define JL_GC_DISABLED __attribute__((annotate("julia_gc_disabled")))
-#define JL_ALWAYS_LEAFTYPE __attribute__((annotate("julia_always_leaftype")))
+#define JL_ALWAYS_LEAFTYPE JL_GLOBALLY_ROOTED
 #define JL_ROOTS_TEMPORARILY __attribute__((annotate("julia_temporarily_roots")))
 #define JL_REQUIRE_ROOTED_SLOT __attribute__((annotate("julia_require_rooted_slot")))
+#define JL_ROOTED_VALUE_COLLECTION __attribute__((annotate("julia_rooted_value_collection")))
 #ifdef __cplusplus
 extern "C" {
 #endif
-  void JL_GC_PROMISE_ROOTED(void *v);
-  void jl_may_leak(uintptr_t);
+  void JL_GC_PROMISE_ROOTED(void *v) JL_NOTSAFEPOINT;
+  void jl_may_leak(uintptr_t) JL_NOTSAFEPOINT;
 #ifdef __cplusplus
 }
 #endif
@@ -33,6 +42,7 @@ extern "C" {
 #define JL_ALWAYS_LEAFTYPE
 #define JL_ROOTS_TEMPORARILY
 #define JL_REQUIRE_ROOTED_SLOT
+#define JL_ROOTED_VALUE_COLLECTION
 #define JL_GC_PROMISE_ROOTED(x) (void)(x)
 #define jl_may_leak(x) (void)(x)
 
