@@ -1,4 +1,4 @@
-// This file is a part of Julia. License is MIT: http://julialang.org/license
+// This file is a part of Julia. License is MIT: https://julialang.org/license
 
 /*
   Hashing
@@ -14,20 +14,7 @@
 extern "C" {
 #endif
 
-uint_t nextipow2(uint_t i)
-{
-    if (i==0) return 1;
-    if ((i&(i-1))==0) return i;
-    if (i&TOP_BIT) return TOP_BIT;
-
-    // repeatedly clear bottom bit
-    while (i&(i-1))
-        i = i&(i-1);
-
-    return i<<1;
-}
-
-u_int32_t int32hash(u_int32_t a)
+uint32_t int32hash(uint32_t a)
 {
     a = (a+0x7ed55d16) + (a<<12);
     a = (a^0xc761c23c) ^ (a>>19);
@@ -38,7 +25,7 @@ u_int32_t int32hash(u_int32_t a)
     return a;
 }
 
-u_int64_t int64hash(u_int64_t key)
+uint64_t int64hash(uint64_t key)
 {
     key = (~key) + (key << 21);            // key = (key << 21) - key - 1;
     key =   key  ^ (key >> 24);
@@ -50,7 +37,7 @@ u_int64_t int64hash(u_int64_t key)
     return key;
 }
 
-u_int32_t int64to32hash(u_int64_t key)
+uint32_t int64to32hash(uint64_t key)
 {
     key = (~key) + (key << 18); // key = (key << 18) - key - 1;
     key =   key  ^ (key >> 31);
@@ -58,7 +45,7 @@ u_int32_t int64to32hash(u_int64_t key)
     key = key ^ (key >> 11);
     key = key + (key << 6);
     key = key ^ (key >> 22);
-    return (u_int32_t)key;
+    return (uint32_t)key;
 }
 
 #include "MurmurHash3.c"
