@@ -22,11 +22,14 @@
  *          _OS_LINUX_
  *          _OS_WINDOWS_
  *          _OS_DARWIN_
+ *          _OS_EMSCRIPTEN_
  *
  *      CPU/Architecture:
  *          _CPU_X86_
  *          _CPU_X86_64_
+ *          _CPU_AARCH64_
  *          _CPU_ARM_
+ *          _CPU_WASM_
  */
 
 /*******************************************************************************
@@ -71,6 +74,8 @@
 #define _OS_WINDOWS_
 #elif defined(__APPLE__) && defined(__MACH__)
 #define _OS_DARWIN_
+#elif defined(__EMSCRIPTEN__)
+#define _OS_EMSCRIPTEN_
 #endif
 
 /*******************************************************************************
@@ -89,6 +94,8 @@
 #define _CPU_PPC64_
 #elif defined(_ARCH_PPC)
 #define _CPU_PPC_
+#elif defined(__wasm__)
+#define _CPU_WASM_
 #endif
 
 #if defined(_CPU_X86_64_)
