@@ -8,7 +8,7 @@
 #define keyhash(k) jl_object_id(k)
 #define h2index(hv, sz) (size_t)(((hv) & ((sz)-1)) * 2)
 
-static int jl_table_assign_bp(jl_array_t **pa, void *key, void *val);
+static int jl_table_assign_bp(jl_array_t **pa, jl_value_t *key, jl_value_t *val);
 
 JL_DLLEXPORT jl_array_t *jl_idtable_rehash(jl_array_t *a, size_t newsz)
 {
@@ -16,7 +16,7 @@ JL_DLLEXPORT jl_array_t *jl_idtable_rehash(jl_array_t *a, size_t newsz)
     // pa doesn't have to be a GC slot but *pa needs to be rooted
     size_t sz = jl_array_len(a);
     size_t i;
-    void **ol = (void **)a->data;
+    jl_value_t **ol = (jl_value_t **)a->data;
     jl_array_t *newa = jl_alloc_vec_any(newsz);
     // keep the original array in the original slot since we need `ol`
     // to be valid in the loop below.
@@ -36,7 +36,7 @@ JL_DLLEXPORT jl_array_t *jl_idtable_rehash(jl_array_t *a, size_t newsz)
     return newa;
 }
 
-static int jl_table_assign_bp(jl_array_t **pa, void *key, void *val)
+static int jl_table_assign_bp(jl_array_t **pa, jl_value_t *key, jl_value_t *val)
 {
     // pa points to a **rooted** gc frame slot
     uint_t hv;
@@ -47,7 +47,7 @@ static int jl_table_assign_bp(jl_array_t **pa, void *key, void *val)
     size_t maxprobe = max_probe(sz);
     void **tab = (void **)a->data;
 
-    hv = keyhash((jl_value_t *)key);
+    hv = keyhash(key);
     while (1) {
         iter = 0;
         index = h2index(hv, sz);
@@ -61,7 +61,7 @@ static int jl_table_assign_bp(jl_array_t **pa, void *key, void *val)
                     empty_slot = index;
                 break;
             }
-            if (jl_egal((jl_value_t *)key, (jl_value_t *)tab[index])) {
+            if (jl_egal(key, (jl_value_t *)tab[index])) {
                 if (tab[index + 1] != NULL) {
                     tab[index + 1] = val;
                     jl_gc_wb(a, val);
@@ -110,13 +110,13 @@ static int jl_table_assign_bp(jl_array_t **pa, void *key, void *val)
 }
 
 /* returns bp if key is in hash, otherwise NULL */
-static void **jl_table_peek_bp(jl_array_t *a, void *key)
+static void **jl_table_peek_bp(jl_array_t *a, jl_value_t *key)
 {
     size_t sz = hash_size(a);
     assert(sz >= 1);
     size_t maxprobe = max_probe(sz);
     void **tab = (void **)a->data;
-    uint_t hv = keyhash((jl_value_t *)key);
+    uint_t hv = keyhash(key);
     size_t index = h2index(hv, sz);
     sz *= 2;
     size_t orig = index;
@@ -125,7 +125,7 @@ static void **jl_table_peek_bp(jl_array_t *a, void *key)
     do {
         if (tab[index] == NULL)
             return NULL;
-        if (jl_egal((jl_value_t *)key, (jl_value_t *)tab[index])) {
+        if (jl_egal(key, (jl_value_t *)tab[index])) {
             if (tab[index + 1] != NULL)
                 return &tab[index + 1];
             // `nothing` is our sentinel value for deletion, so need to keep searching if it's also our search key
@@ -140,7 +140,7 @@ static void **jl_table_peek_bp(jl_array_t *a, void *key)
 }
 
 JL_DLLEXPORT
-jl_array_t *jl_eqtable_put(jl_array_t *h, void *key, void *val, int *p_inserted)
+jl_array_t *jl_eqtable_put(jl_array_t *h, jl_value_t *key, jl_value_t *val, int *p_inserted)
 {
     JL_GC_PUSH1(&h);
     // &h may be assigned to in jl_idtable_rehash so it need to be rooted
@@ -152,14 +152,14 @@ jl_array_t *jl_eqtable_put(jl_array_t *h, void *key, void *val, int *p_inserted)
 }
 
 JL_DLLEXPORT
-jl_value_t *jl_eqtable_get(jl_array_t *h, void *key, jl_value_t *deflt)
+jl_value_t *jl_eqtable_get(jl_array_t *h, jl_value_t *key, jl_value_t *deflt)
 {
     void **bp = jl_table_peek_bp(h, key);
     return (bp == NULL) ? deflt : (jl_value_t *)*bp;
 }
 
 JL_DLLEXPORT
-jl_value_t *jl_eqtable_pop(jl_array_t *h, void *key, jl_value_t *deflt, int *found)
+jl_value_t *jl_eqtable_pop(jl_array_t *h, jl_value_t *key, jl_value_t *deflt, int *found)
 {
     void **bp = jl_table_peek_bp(h, key);
     if (found)
