@@ -27,17 +27,12 @@ extern "C" {
 JL_DLLEXPORT int16_t jl_threadid(void);
 JL_DLLEXPORT void jl_threading_profile(void);
 
-JL_DLLEXPORT JL_CONST_FUNC jl_ptls_t (jl_get_ptls_states)(void);
+JL_DLLEXPORT JL_CONST_FUNC jl_ptls_t (jl_get_ptls_states)(void) JL_GLOBALLY_ROOTED JL_NOTSAFEPOINT;
 
-#ifndef JULIA_ENABLE_THREADING
-extern JL_DLLEXPORT jl_tls_states_t jl_tls_states;
-#define jl_get_ptls_states() (&jl_tls_states)
-#else // ifndef JULIA_ENABLE_THREADING
 typedef jl_ptls_t (*jl_get_ptls_states_func)(void);
 #if !defined(_OS_DARWIN_) && !defined(_OS_WINDOWS_)
 JL_DLLEXPORT void jl_set_ptls_states_getter(jl_get_ptls_states_func f);
 #endif
-#endif
 
 #ifdef __cplusplus
 }
