@@ -34,7 +34,7 @@ All subtypes of `AbstractDateToken` must define this method in order
 to be able to print a Date / DateTime object according to a `DateFormat`
 containing that token.
 """
-function format end
+format(io::IO, tok::AbstractDateToken, dt::TimeType, locale)
 
 # fallback to tryparsenext/format methods that don't care about locale
 @inline function tryparsenext(d::AbstractDateToken, str, i, len, locale)
@@ -51,7 +51,31 @@ function Base.string(t::Time)
     return "$hh:$mii:$ss$ns"
 end
 
-Base.show(io::IO, x::Time) = print(io, string(x))
+Base.show(io::IO, ::MIME"text/plain", t::Time) = print(io, t)
+Base.print(io::IO, t::Time) = print(io, string(t))
+
+function Base.show(io::IO, t::Time)
+    if get(io, :compact, false)
+        print(io, t)
+    else
+        values = [
+            hour(t)
+            minute(t)
+            second(t)
+            millisecond(t)
+            microsecond(t)
+            nanosecond(t)
+        ]
+        index = something(findlast(!iszero, values), 1)
+
+        print(io, Time, "(")
+        for i in 1:index
+            show(io, values[i])
+            i != index && print(io, ", ")
+        end
+        print(io, ")")
+    end
+end
 
 @inline function format(io, d::AbstractDateToken, dt, locale)
     format(io, d, dt)
@@ -87,7 +111,7 @@ end
 
 ### Parse tokens
 
-for c in "yYmdHMS"
+for c in "yYmdHIMS"
     @eval begin
         @inline function tryparsenext(d::DatePart{$c}, str, i, len)
             return tryparsenext_base10(str, i, len, min_width(d), max_width(d))
@@ -95,6 +119,16 @@ for c in "yYmdHMS"
     end
 end
 
+function tryparsenext(d::DatePart{'p'}, str, i, len)
+    i+1 > len && return nothing
+    c, ii = iterate(str, i)::Tuple{Char, Int}
+    ap = lowercase(c)
+    (ap == 'a' || ap == 'p') || return nothing
+    c, ii = iterate(str, ii)::Tuple{Char, Int}
+    lowercase(c) == 'm' || return nothing
+    return ap == 'a' ? AM : PM, ii
+end
+
 for (tok, fn) in zip("uUeE", [monthabbr_to_value, monthname_to_value, dayabbr_to_value, dayname_to_value])
     @eval @inline function tryparsenext(d::DatePart{$tok}, str, i, len, locale)
         next = tryparsenext_word(str, i, len, locale, max_width(d))
@@ -125,7 +159,9 @@ end
 
 ### Format tokens
 
-for (c, fn) in zip("YmdHMS", [year, month, day, hour, minute, second])
+hour12(dt) = let h = hour(dt); h > 12 ? h - 12 : h == 0 ? 12 : h; end
+
+for (c, fn) in zip("YmdHIMS", [year, month, day, hour, hour12, minute, second])
     @eval function format(io, d::DatePart{$c}, dt)
         print(io, string($fn(dt), base = 10, pad = d.width))
     end
@@ -137,6 +173,11 @@ for (tok, fn) in zip("uU", [monthabbr, monthname])
     end
 end
 
+function format(io, d::DatePart{'p'}, dt, locale)
+    ampm = hour(dt) < 12 ? "AM" : "PM" # fixme: locale-specific?
+    print(io, ampm)
+end
+
 for (tok, fn) in zip("eE", [dayabbr, dayname])
     @eval function format(io, ::DatePart{$tok}, dt, locale)
         print(io, $fn(dayofweek(dt), locale))
@@ -259,9 +300,11 @@ const CONVERSION_SPECIFIERS = Dict{Char, Type}(
     'E' => DayOfWeekToken,
     'd' => Day,
     'H' => Hour,
+    'I' => Hour,
     'M' => Minute,
     'S' => Second,
     's' => Millisecond,
+    'p' => AMPM,
 )
 
 # Default values are needed when a conversion specifier is used in a DateFormat for parsing
@@ -278,14 +321,15 @@ const CONVERSION_DEFAULTS = IdDict{Type, Any}(
     Millisecond => Int64(0),
     Microsecond => Int64(0),
     Nanosecond => Int64(0),
+    AMPM => TWENTYFOURHOUR,
 )
 
 # Specifies the required fields in order to parse a TimeType
 # Note: Allows for addition of new TimeTypes
 const CONVERSION_TRANSLATIONS = IdDict{Type, Any}(
     Date => (Year, Month, Day),
-    DateTime => (Year, Month, Day, Hour, Minute, Second, Millisecond),
-    Time => (Hour, Minute, Second, Millisecond, Microsecond, Nanosecond),
+    DateTime => (Year, Month, Day, Hour, Minute, Second, Millisecond, AMPM),
+    Time => (Hour, Minute, Second, Millisecond, Microsecond, Nanosecond, AMPM),
 )
 
 """
@@ -303,12 +347,14 @@ string:
 | `u`        | Jan       | Matches abbreviated months according to the `locale` keyword |
 | `U`        | January   | Matches full month names according to the `locale` keyword   |
 | `d`        | 1, 01     | Matches 1 or 2-digit days                                    |
-| `H`        | 00        | Matches hours                                                |
+| `H`        | 00        | Matches hours (24-hour clock)                                |
+| `I`        | 00        | For outputting hours with 12-hour clock                      |
 | `M`        | 00        | Matches minutes                                              |
 | `S`        | 00        | Matches seconds                                              |
 | `s`        | .500      | Matches milliseconds                                         |
 | `e`        | Mon, Tues | Matches abbreviated days of the week                         |
 | `E`        | Monday    | Matches full name days of the week                           |
+| `p`        | AM        | Matches AM/PM (case-insensitive)                             |
 | `yyyymmdd` | 19960101  | Matches fixed-width year, month, and day                     |
 
 Characters not listed above are normally treated as delimiters between date and time slots.
@@ -316,6 +362,10 @@ For example a `dt` string of "1996-01-15T00:00:00.0" would have a `format` strin
 "y-m-dTH:M:S.s". If you need to use a code character as a delimiter you can escape it using
 backslash. The date "1995y01m" would have the format "y\\ym\\m".
 
+Note that 12:00AM corresponds 00:00 (midnight), and 12:00PM corresponds to 12:00 (noon).
+When parsing a time with a `p` specifier, any hour (either `H` or `I`) is interpreted as
+as a 12-hour clock, so the `I` code is mainly useful for output.
+
 Creating a DateFormat object is expensive. Whenever possible, create it once and use it many times
 or try the `dateformat""` string macro. Using this macro creates the DateFormat object once at
 macro expansion time and reuses it later. see [`@dateformat_str`](@ref).
