@@ -3,13 +3,57 @@
 ####################
 # LU Factorization #
 ####################
+"""
+    LU <: Factorization
+
+Matrix factorization type of the `LU` factorization of a square matrix `A`. This
+is the return type of [`lu`](@ref), the corresponding matrix factorization function.
+
+The individual components of the factorization `F::LU` can be accessed via `getproperty`:
+
+| Component | Description                              |
+|:----------|:-----------------------------------------|
+| `F.L`     | `L` (unit lower triangular) part of `LU` |
+| `F.U`     | `U` (upper triangular) part of `LU`      |
+| `F.p`     | (right) permutation `Vector`             |
+| `F.P`     | (right) permutation `Matrix`             |
+
+Iterating the factorization produces the components `F.L`, `F.U`, and `F.p`.
+
+# Examples
+```jldoctest
+julia> A = [4 3; 6 3]
+2×2 Array{Int64,2}:
+ 4  3
+ 6  3
+
+julia> F = lu(A)
+LU{Float64,Array{Float64,2}}
+L factor:
+2×2 Array{Float64,2}:
+ 1.0       0.0
+ 0.666667  1.0
+U factor:
+2×2 Array{Float64,2}:
+ 6.0  3.0
+ 0.0  1.0
+
+julia> F.L * F.U == A[F.p, :]
+true
+
+julia> l, u, p = lu(A); # destructuring via iteration
+
+julia> l == F.L && u == F.U && p == F.p
+true
+```
+"""
 struct LU{T,S<:AbstractMatrix{T}} <: Factorization{T}
     factors::S
     ipiv::Vector{BlasInt}
     info::BlasInt
 
     function LU{T,S}(factors, ipiv, info) where {T,S<:AbstractMatrix{T}}
-        @assert !has_offset_axes(factors)
+        require_one_based_indexing(factors)
         new{T,S}(factors, ipiv, info)
     end
 end
@@ -131,17 +175,10 @@ function generic_lufact!(A::StridedMatrix{T}, ::Val{Pivot} = Val(true);
             end
         end
     end
-    check && checknonsingular(info)
+    check && checknonsingular(info, Val{Pivot}())
     return LU{T,typeof(A)}(A, ipiv, convert(BlasInt, info))
 end
 
-# floating point types doesn't have to be promoted for LU, but should default to pivoting
-function lu(A::Union{AbstractMatrix{T}, AbstractMatrix{Complex{T}}},
-            pivot::Union{Val{false}, Val{true}} = Val(true);
-            check::Bool = true) where {T<:AbstractFloat}
-    lu!(copy(A), pivot; check = check)
-end
-
 function lutype(T::Type)
     # In generic_lufact!, the elements of the lower part of the matrix are
     # obtained using the division of two matrix elements. Hence their type can
@@ -214,12 +251,12 @@ julia> F = lu(A)
 LU{Float64,Array{Float64,2}}
 L factor:
 2×2 Array{Float64,2}:
- 1.0  0.0
- 1.5  1.0
+ 1.0       0.0
+ 0.666667  1.0
 U factor:
 2×2 Array{Float64,2}:
- 4.0   3.0
- 0.0  -1.5
+ 6.0  3.0
+ 0.0  1.0
 
 julia> F.L * F.U == A[F.p, :]
 true
@@ -230,26 +267,10 @@ julia> l == F.L && u == F.U && p == F.p
 true
 ```
 """
-function lu(A::AbstractMatrix{T}, pivot::Union{Val{false}, Val{true}};
+function lu(A::AbstractMatrix{T}, pivot::Union{Val{false}, Val{true}}=Val(true);
             check::Bool = true) where T
     S = lutype(T)
-    AA = similar(A, S)
-    copyto!(AA, A)
-    lu!(AA, pivot; check = check)
-end
-# We can't assume an ordered field so we first try without pivoting
-function lu(A::AbstractMatrix{T}; check::Bool = true) where T
-    S = lutype(T)
-    AA = similar(A, S)
-    copyto!(AA, A)
-    F = lu!(AA, Val(false); check = false)
-    if issuccess(F)
-        return F
-    else
-        AA = similar(A, S)
-        copyto!(AA, A)
-        return lu!(AA, Val(true); check = check)
-    end
+    lu!(copy_oftype(A, S), pivot; check = check)
 end
 
 lu(S::LU) = S
@@ -273,7 +294,7 @@ size(A::LU)    = size(getfield(A, :factors))
 size(A::LU, i) = size(getfield(A, :factors), i)
 
 function ipiv2perm(v::AbstractVector{T}, maxi::Integer) where T
-    @assert !has_offset_axes(v)
+    require_one_based_indexing(v)
     p = T[1:maxi;]
     @inbounds for i in 1:length(v)
         p[i], p[v[i]] = p[v[i]], p[i]
@@ -283,15 +304,15 @@ end
 
 function getproperty(F::LU{T,<:StridedMatrix}, d::Symbol) where T
     m, n = size(F)
-    if d == :L
+    if d === :L
         L = tril!(getfield(F, :factors)[1:m, 1:min(m,n)])
         for i = 1:min(m,n); L[i,i] = one(T); end
         return L
-    elseif d == :U
+    elseif d === :U
         return triu!(getfield(F, :factors)[1:min(m,n), 1:n])
-    elseif d == :p
+    elseif d === :p
         return ipiv2perm(getfield(F, :ipiv), m)
-    elseif d == :P
+    elseif d === :P
         return Matrix{T}(I, m, m)[:,invperm(F.p)]
     else
         getfield(F, d)
@@ -315,10 +336,10 @@ function show(io::IO, mime::MIME{Symbol("text/plain")}, F::LU)
     end
 end
 
-_apply_ipiv!(A::LU, B::StridedVecOrMat) = _ipiv!(A, 1 : length(A.ipiv), B)
-_apply_inverse_ipiv!(A::LU, B::StridedVecOrMat) = _ipiv!(A, length(A.ipiv) : -1 : 1, B)
+_apply_ipiv_rows!(A::LU, B::StridedVecOrMat) = _ipiv_rows!(A, 1 : length(A.ipiv), B)
+_apply_inverse_ipiv_rows!(A::LU, B::StridedVecOrMat) = _ipiv_rows!(A, length(A.ipiv) : -1 : 1, B)
 
-function _ipiv!(A::LU, order::OrdinalRange, B::StridedVecOrMat)
+function _ipiv_rows!(A::LU, order::OrdinalRange, B::StridedVecOrMat)
     for i = order
         if i != A.ipiv[i]
             _swap_rows!(B, i, A.ipiv[i])
@@ -339,11 +360,39 @@ function _swap_rows!(B::StridedMatrix, i::Integer, j::Integer)
     B
 end
 
+_apply_ipiv_cols!(A::LU, B::StridedVecOrMat) = _ipiv_cols!(A, 1 : length(A.ipiv), B)
+_apply_inverse_ipiv_cols!(A::LU, B::StridedVecOrMat) = _ipiv_cols!(A, length(A.ipiv) : -1 : 1, B)
+
+function _ipiv_cols!(A::LU, order::OrdinalRange, B::StridedVecOrMat)
+    for i = order
+        if i != A.ipiv[i]
+            _swap_cols!(B, i, A.ipiv[i])
+        end
+    end
+    B
+end
+
+function _swap_cols!(B::StridedVector, i::Integer, j::Integer)
+    _swap_rows!(B, i, j)
+end
+
+function _swap_cols!(B::StridedMatrix, i::Integer, j::Integer)
+    for row = 1 : size(B, 1)
+        B[row,i], B[row,j] = B[row,j], B[row,i]
+    end
+    B
+end
+
+function rdiv!(A::StridedVecOrMat, B::LU{<:Any,<:StridedMatrix})
+    rdiv!(rdiv!(A, UpperTriangular(B.factors)), UnitLowerTriangular(B.factors))
+    _apply_inverse_ipiv_cols!(B, A)
+end
+
 ldiv!(A::LU{T,<:StridedMatrix}, B::StridedVecOrMat{T}) where {T<:BlasFloat} =
     LAPACK.getrs!('N', A.factors, A.ipiv, B)
 
 function ldiv!(A::LU{<:Any,<:StridedMatrix}, B::StridedVecOrMat)
-    _apply_ipiv!(A, B)
+    _apply_ipiv_rows!(A, B)
     ldiv!(UpperTriangular(A.factors), ldiv!(UnitLowerTriangular(A.factors), B))
 end
 
@@ -353,7 +402,7 @@ ldiv!(transA::Transpose{T,<:LU{T,<:StridedMatrix}}, B::StridedVecOrMat{T}) where
 function ldiv!(transA::Transpose{<:Any,<:LU{<:Any,<:StridedMatrix}}, B::StridedVecOrMat)
     A = transA.parent
     ldiv!(transpose(UnitLowerTriangular(A.factors)), ldiv!(transpose(UpperTriangular(A.factors)), B))
-    _apply_inverse_ipiv!(A, B)
+    _apply_inverse_ipiv_rows!(A, B)
 end
 
 ldiv!(adjF::Adjoint{T,<:LU{T,<:StridedMatrix}}, B::StridedVecOrMat{T}) where {T<:Real} =
@@ -364,16 +413,32 @@ ldiv!(adjA::Adjoint{T,<:LU{T,<:StridedMatrix}}, B::StridedVecOrMat{T}) where {T<
 function ldiv!(adjA::Adjoint{<:Any,<:LU{<:Any,<:StridedMatrix}}, B::StridedVecOrMat)
     A = adjA.parent
     ldiv!(adjoint(UnitLowerTriangular(A.factors)), ldiv!(adjoint(UpperTriangular(A.factors)), B))
-    _apply_inverse_ipiv!(A, B)
+    _apply_inverse_ipiv_rows!(A, B)
 end
 
-\(A::Adjoint{<:Any,<:LU}, B::Adjoint{<:Any,<:StridedVecOrMat}) = A \ copy(B)
-\(A::Transpose{<:Any,<:LU}, B::Transpose{<:Any,<:StridedVecOrMat}) = A \ copy(B)
-\(A::Adjoint{T,<:LU{T,<:StridedMatrix}}, B::Adjoint{T,<:StridedVecOrMat{T}}) where {T<:BlasComplex} =
+(\)(A::Adjoint{<:Any,<:LU}, B::Adjoint{<:Any,<:StridedVecOrMat}) = A \ copy(B)
+(\)(A::Transpose{<:Any,<:LU}, B::Transpose{<:Any,<:StridedVecOrMat}) = A \ copy(B)
+(\)(A::Adjoint{T,<:LU{T,<:StridedMatrix}}, B::Adjoint{T,<:StridedVecOrMat{T}}) where {T<:BlasComplex} =
     LAPACK.getrs!('C', A.parent.factors, A.parent.ipiv, copy(B))
-\(A::Transpose{T,<:LU{T,<:StridedMatrix}}, B::Transpose{T,<:StridedVecOrMat{T}}) where {T<:BlasFloat} =
+(\)(A::Transpose{T,<:LU{T,<:StridedMatrix}}, B::Transpose{T,<:StridedVecOrMat{T}}) where {T<:BlasFloat} =
     LAPACK.getrs!('T', A.parent.factors, A.parent.ipiv, copy(B))
 
+function (/)(A::AbstractMatrix, F::Adjoint{<:Any,<:LU})
+    T = promote_type(eltype(A), eltype(F))
+    return adjoint(ldiv!(F.parent, copy_oftype(adjoint(A), T)))
+end
+# To avoid ambiguities with definitions in adjtrans.jl and factorizations.jl
+(/)(adjA::Adjoint{<:Any,<:AbstractVector}, F::Adjoint{<:Any,<:LU}) = adjoint(F.parent \ adjA.parent)
+(/)(adjA::Adjoint{<:Any,<:AbstractMatrix}, F::Adjoint{<:Any,<:LU}) = adjoint(F.parent \ adjA.parent)
+function (/)(trA::Transpose{<:Any,<:AbstractVector}, F::Adjoint{<:Any,<:LU})
+    T = promote_type(eltype(trA), eltype(F))
+    return adjoint(ldiv!(F.parent, convert(AbstractVector{T}, conj(trA.parent))))
+end
+function (/)(trA::Transpose{<:Any,<:AbstractMatrix}, F::Adjoint{<:Any,<:LU})
+    T = promote_type(eltype(trA), eltype(F))
+    return adjoint(ldiv!(F.parent, convert(AbstractMatrix{T}, conj(trA.parent))))
+end
+
 function det(F::LU{T}) where T
     n = checksquare(F)
     issuccess(F) || return zero(T)
@@ -413,13 +478,6 @@ inv!(A::LU{T,<:StridedMatrix}) where {T} =
     ldiv!(A.factors, copy(A), Matrix{T}(I, size(A, 1), size(A, 1)))
 inv(A::LU{<:BlasFloat,<:StridedMatrix}) = inv!(copy(A))
 
-function _cond1Inf(A::LU{<:BlasFloat,<:StridedMatrix}, p::Number, normA::Real)
-    if p != 1 && p != Inf
-        throw(ArgumentError("p must be either 1 or Inf"))
-    end
-    return inv(LAPACK.gecon!(p == 1 ? '1' : 'I', A.factors, normA))
-end
-
 # Tridiagonal
 
 # See dgttrf.f
@@ -487,7 +545,7 @@ function lu!(A::Tridiagonal{T,V}, pivot::Union{Val{false}, Val{true}} = Val(true
         end
     end
     B = Tridiagonal{T,V}(dl, d, du, du2)
-    check && checknonsingular(info)
+    check && checknonsingular(info, pivot)
     return LU{T,Tridiagonal{T,V}}(B, ipiv, convert(BlasInt, info))
 end
 
@@ -495,7 +553,7 @@ factorize(A::Tridiagonal) = lu(A)
 
 function getproperty(F::LU{T,Tridiagonal{T,V}}, d::Symbol) where {T,V}
     m, n = size(F)
-    if d == :L
+    if d === :L
         dl = getfield(getfield(F, :factors), :dl)
         L = Array(Bidiagonal(fill!(similar(dl, n), one(T)), dl, d))
         for i = 2:n
@@ -504,15 +562,15 @@ function getproperty(F::LU{T,Tridiagonal{T,V}}, d::Symbol) where {T,V}
             L[i, 1:i - 1] = tmp
         end
         return L
-    elseif d == :U
+    elseif d === :U
         U = Array(Bidiagonal(getfield(getfield(F, :factors), :d), getfield(getfield(F, :factors), :du), d))
         for i = 1:n - 2
             U[i,i + 2] = getfield(getfield(F, :factors), :du2)[i]
         end
         return U
-    elseif d == :p
+    elseif d === :p
         return ipiv2perm(getfield(F, :ipiv), m)
-    elseif d == :P
+    elseif d === :P
         return Matrix{T}(I, m, m)[:,invperm(F.p)]
     end
     return getfield(F, d)
@@ -520,7 +578,7 @@ end
 
 # See dgtts2.f
 function ldiv!(A::LU{T,Tridiagonal{T,V}}, B::AbstractVecOrMat) where {T,V}
-    @assert !has_offset_axes(B)
+    require_one_based_indexing(B)
     n = size(A,1)
     if n != size(B,1)
         throw(DimensionMismatch("matrix has dimensions ($n,$n) but right hand side has $(size(B,1)) rows"))
@@ -552,7 +610,7 @@ function ldiv!(A::LU{T,Tridiagonal{T,V}}, B::AbstractVecOrMat) where {T,V}
 end
 
 function ldiv!(transA::Transpose{<:Any,<:LU{T,Tridiagonal{T,V}}}, B::AbstractVecOrMat) where {T,V}
-    @assert !has_offset_axes(B)
+    require_one_based_indexing(B)
     A = transA.parent
     n = size(A,1)
     if n != size(B,1)
@@ -589,7 +647,7 @@ end
 
 # Ac_ldiv_B!(A::LU{T,Tridiagonal{T}}, B::AbstractVecOrMat) where {T<:Real} = At_ldiv_B!(A,B)
 function ldiv!(adjA::Adjoint{<:Any,LU{T,Tridiagonal{T,V}}}, B::AbstractVecOrMat) where {T,V}
-    @assert !has_offset_axes(B)
+    require_one_based_indexing(B)
     A = adjA.parent
     n = size(A,1)
     if n != size(B,1)
@@ -624,7 +682,9 @@ function ldiv!(adjA::Adjoint{<:Any,LU{T,Tridiagonal{T,V}}}, B::AbstractVecOrMat)
     return B
 end
 
-/(B::AbstractMatrix, A::LU) = copy(transpose(transpose(A) \ transpose(B)))
+rdiv!(B::AbstractMatrix, A::LU) = transpose(ldiv!(transpose(A), transpose(B)))
+rdiv!(B::AbstractMatrix, A::Transpose{<:Any,<:LU}) = transpose(ldiv!(A.parent, transpose(B)))
+rdiv!(B::AbstractMatrix, A::Adjoint{<:Any,<:LU}) = adjoint(ldiv!(A.parent, adjoint(B)))
 
 # Conversions
 AbstractMatrix(F::LU) = (F.L * F.U)[invperm(F.p),:]
