@@ -40,7 +40,6 @@ struct MethodCompletion <: Completion
     func
     input_types::Type
     method::Method
-    kwtype # needed for printing
 end
 
 struct BslashCompletion <: Completion
@@ -62,7 +61,7 @@ completion_text(c::ModuleCompletion) = c.mod
 completion_text(c::PackageCompletion) = c.package
 completion_text(c::PropertyCompletion) = string(c.property)
 completion_text(c::FieldCompletion) = string(c.field)
-completion_text(c::MethodCompletion) = sprint(io -> show(io, c.method, kwtype=c.kwtype))
+completion_text(c::MethodCompletion) = sprint(io -> show(io, c.method))
 completion_text(c::BslashCompletion) = c.bslash
 completion_text(c::ShellCompletion) = c.text
 completion_text(c::DictCompletion) = c.key
@@ -185,7 +184,7 @@ function complete_keyword(s::Union{String,SubString{String}})::Vector{Completion
     map(KeywordCompletion, sorted_keywords[r])
 end
 
-function complete_path(path::AbstractString, pos; use_envpath=false)::Completions
+function complete_path(path::AbstractString, pos; use_envpath=false, shell_escape=false)::Completions
     if Base.Sys.isunix() && occursin(r"^~(?:/|$)", path)
         # if the path is just "~", don't consider the expanded username as a prefix
         if path == "~"
@@ -260,7 +259,7 @@ function complete_path(path::AbstractString, pos; use_envpath=false)::Completion
         end
     end
 
-    matchList = Completion[PathCompletion(replace(s, r"\s" => "\\ ")) for s in matches]
+    matchList = PathCompletion[PathCompletion(shell_escape ? replace(s, r"\s" => s"\\\0") : s) for s in matches]
     startpos = pos - lastindex(prefix) + 1 - count(isequal(' '), prefix)
     # The pos - lastindex(prefix) + 1 is correct due to `lastindex(prefix)-lastindex(prefix)==0`,
     # hence we need to add one to get the first index. This is also correct when considering
@@ -451,13 +450,12 @@ function complete_methods(ex_org::Expr, context_module=Main)::Vector{Completion}
     t_in = Tuple{Core.Typeof(func), args_ex...} # Input types
     na = length(args_ex)+1
     ml = methods(func)
-    kwtype = isdefined(ml.mt, :kwsorter) ? typeof(ml.mt.kwsorter) : nothing
     for method in ml
         ms = method.sig
 
         # Check if the method's type signature intersects the input types
         if typeintersect(Base.rewrap_unionall(Tuple{Base.unwrap_unionall(ms).parameters[1 : min(na, end)]...}, ms), t_in) != Union{}
-            push!(out, MethodCompletion(func, t_in, method, kwtype))
+            push!(out, MethodCompletion(func, t_in, method))
         end
     end
     return out
@@ -539,7 +537,7 @@ end
 
 # This needs to be a separate non-inlined function, see #19441
 @noinline function find_dict_matches(identifier, partial_key)
-    matches = []
+    matches = String[]
     for key in keys(identifier)
         rkey = repr(key)
         startswith(rkey,partial_key) && push!(matches,rkey)
@@ -730,7 +728,7 @@ function shell_completions(string, pos)::Completions
         # Also try looking into the env path if the user wants to complete the first argument
         use_envpath = !ignore_last_word && length(args.args) < 2
 
-        return complete_path(prefix, pos, use_envpath=use_envpath)
+        return complete_path(prefix, pos, use_envpath=use_envpath, shell_escape=true)
     elseif isexpr(arg, :incomplete) || isexpr(arg, :error)
         partial = scs[last_parse]
         ret, range = completions(partial, lastindex(partial))
