@@ -1,5 +1,8 @@
 # This file is a part of Julia. License is MIT: https://julialang.org/license
 
+"""
+Support for sockets. Provides [`IPAddr`](@ref) and subtypes, [`TCPSocket`](@ref), and [`UDPSocket`](@ref).
+"""
 module Sockets
 
 export
@@ -10,6 +13,7 @@ export
     getalladdrinfo,
     getnameinfo,
     getipaddr,
+    getipaddrs,
     getpeername,
     getsockname,
     listen,
@@ -27,9 +31,10 @@ export
 import Base: isless, show, print, parse, bind, convert, isreadable, iswritable, alloc_buf_hook, _uv_hook_close
 
 using Base: LibuvStream, LibuvServer, PipeEndpoint, @handle_as, uv_error, associate_julia_struct, uvfinalize,
-    notify_error, stream_wait, uv_req_data, uv_req_set_data, preserve_handle, unpreserve_handle, _UVError, IOError,
+    notify_error, uv_req_data, uv_req_set_data, preserve_handle, unpreserve_handle, _UVError, IOError,
     eventloop, StatusUninit, StatusInit, StatusConnecting, StatusOpen, StatusClosing, StatusClosed, StatusActive,
-    uv_status_string, check_open, wait_connected,
+    preserve_handle, unpreserve_handle, iolock_begin, iolock_end,
+    uv_status_string, check_open, OS_HANDLE, RawFD,
     UV_EINVAL, UV_ENOMEM, UV_ENOBUFS, UV_EAGAIN, UV_ECONNABORTED, UV_EADDRINUSE, UV_EACCES, UV_EADDRNOTAVAIL,
     UV_EAI_ADDRFAMILY, UV_EAI_AGAIN, UV_EAI_BADFLAGS,
     UV_EAI_BADHINTS, UV_EAI_CANCELED, UV_EAI_FAIL,
@@ -51,11 +56,10 @@ mutable struct TCPSocket <: LibuvStream
     handle::Ptr{Cvoid}
     status::Int
     buffer::IOBuffer
-    readnotify::Condition
-    connectnotify::Condition
-    closenotify::Condition
+    cond::Base.ThreadSynchronizer
+    readerror::Any
     sendbuf::Union{IOBuffer, Nothing}
-    lock::ReentrantLock
+    lock::ReentrantLock # advisory lock
     throttle::Int
 
     function TCPSocket(handle::Ptr{Cvoid}, status)
@@ -63,9 +67,8 @@ mutable struct TCPSocket <: LibuvStream
                 handle,
                 status,
                 PipeBuffer(),
-                Condition(),
-                Condition(),
-                Condition(),
+                Base.ThreadSynchronizer(),
+                nothing,
                 nothing,
                 ReentrantLock(),
                 Base.DEFAULT_READ_BUFFER_SZ)
@@ -79,25 +82,39 @@ end
 function TCPSocket(; delay=true)
     tcp = TCPSocket(Libc.malloc(Base._sizeof_uv_tcp), StatusUninit)
     af_spec = delay ? 0 : 2   # AF_UNSPEC is 0, AF_INET is 2
+    iolock_begin()
     err = ccall(:uv_tcp_init_ex, Cint, (Ptr{Cvoid}, Ptr{Cvoid}, Cuint),
                 eventloop(), tcp.handle, af_spec)
     uv_error("failed to create tcp socket", err)
     tcp.status = StatusInit
+    iolock_end()
     return tcp
 end
 
+function TCPSocket(fd::OS_HANDLE)
+    tcp = TCPSocket()
+    iolock_begin()
+    err = ccall(:uv_tcp_open, Int32, (Ptr{Cvoid}, OS_HANDLE), pipe.handle, fd)
+    uv_error("tcp_open", err)
+    tcp.status = StatusOpen
+    iolock_end()
+    return tcp
+end
+if OS_HANDLE != RawFD
+    TCPSocket(fd::RawFD) = TCPSocket(Libc._get_osfhandle(fd))
+end
+
+
 mutable struct TCPServer <: LibuvServer
     handle::Ptr{Cvoid}
     status::Int
-    connectnotify::Condition
-    closenotify::Condition
+    cond::Base.ThreadSynchronizer
 
     function TCPServer(handle::Ptr{Cvoid}, status)
         tcp = new(
             handle,
             status,
-            Condition(),
-            Condition())
+            Base.ThreadSynchronizer())
         associate_julia_struct(tcp.handle, tcp)
         finalizer(uvfinalize, tcp)
         return tcp
@@ -110,10 +127,12 @@ end
 function TCPServer(; delay=true)
     tcp = TCPServer(Libc.malloc(Base._sizeof_uv_tcp), StatusUninit)
     af_spec = delay ? 0 : 2   # AF_UNSPEC is 0, AF_INET is 2
+    iolock_begin()
     err = ccall(:uv_tcp_init_ex, Cint, (Ptr{Cvoid}, Ptr{Cvoid}, Cuint),
                 eventloop(), tcp.handle, af_spec)
     uv_error("failed to create tcp server", err)
     tcp.status = StatusInit
+    iolock_end()
     return tcp
 end
 
@@ -121,7 +140,7 @@ isreadable(io::TCPSocket) = isopen(io) || bytesavailable(io) > 0
 iswritable(io::TCPSocket) = isopen(io) && io.status != StatusClosing
 
 """
-    accept(server[,client])
+    accept(server[, client])
 
 Accepts a connection on the given server and returns a connection to the client. An
 uninitialized client stream may be provided, in which case it will be used instead of
@@ -129,6 +148,23 @@ creating a new stream.
 """
 accept(server::TCPServer) = accept(server, TCPSocket())
 
+function accept(callback, server::LibuvServer)
+    task = @async try
+            while true
+                client = accept(server)
+                callback(client)
+            end
+        catch ex
+            # accept below may explicitly throw UV_ECONNABORTED:
+            # filter that out since we expect that error
+            if !(ex isa IOError && ex.code == UV_ECONNABORTED) || isopen(server)
+                rethrow()
+            end
+        end
+    return task # caller is responsible for checking for errors
+end
+
+
 # UDP
 """
     UDPSocket()
@@ -139,17 +175,12 @@ fields to denote the state of the socket.
 mutable struct UDPSocket <: LibuvStream
     handle::Ptr{Cvoid}
     status::Int
-    recvnotify::Condition
-    sendnotify::Condition
-    closenotify::Condition
+    recvnotify::Base.ThreadSynchronizer
+    cond::Base.ThreadSynchronizer
 
     function UDPSocket(handle::Ptr{Cvoid}, status)
-        udp = new(
-            handle,
-            status,
-            Condition(),
-            Condition(),
-            Condition())
+        cond = Base.ThreadSynchronizer()
+        udp = new(handle, status, Base.ThreadSynchronizer(cond.lock), cond)
         associate_julia_struct(udp.handle, udp)
         finalizer(uvfinalize, udp)
         return udp
@@ -157,10 +188,12 @@ mutable struct UDPSocket <: LibuvStream
 end
 function UDPSocket()
     this = UDPSocket(Libc.malloc(Base._sizeof_uv_udp), StatusUninit)
+    iolock_begin()
     err = ccall(:uv_udp_init, Cint, (Ptr{Cvoid}, Ptr{Cvoid}),
                 eventloop(), this.handle)
     uv_error("failed to create udp socket", err)
     this.status = StatusInit
+    iolock_end()
     return this
 end
 
@@ -168,10 +201,15 @@ show(io::IO, stream::UDPSocket) = print(io, typeof(stream), "(", uv_status_strin
 
 function _uv_hook_close(sock::UDPSocket)
     sock.handle = C_NULL
-    sock.status = StatusClosed
-    notify(sock.closenotify)
-    notify(sock.sendnotify)
-    notify_error(sock.recvnotify,EOFError())
+    lock(sock.cond)
+    try
+        sock.status = StatusClosed
+        notify(sock.cond)
+        notify_error(sock.recvnotify, EOFError())
+    finally
+        unlock(sock.cond)
+    end
+    nothing
 end
 
 # Disables dual stack mode.
@@ -193,17 +231,17 @@ const UV_UDP_REUSEADDR = 4
 
 ##
 
-_bind(sock::TCPServer, host::IPv4, port::UInt16, flags::UInt32 = UInt32(0)) = ccall(:jl_tcp_bind, Int32, (Ptr{Cvoid}, UInt16, UInt32, Cuint),
-            sock.handle, hton(port), hton(host.host), flags)
+function _bind(sock::TCPServer, host::Union{IPv4, IPv6}, port::UInt16, flags::UInt32=UInt32(0))
+    host_in = Ref(hton(host.host))
+    return ccall(:jl_tcp_bind, Int32, (Ptr{Cvoid}, UInt16, Ptr{Cvoid}, Cuint, Cint),
+            sock, hton(port), host_in, flags, host isa IPv6)
+end
 
-_bind(sock::TCPServer, host::IPv6, port::UInt16, flags::UInt32 = UInt32(0)) = ccall(:jl_tcp_bind6, Int32, (Ptr{Cvoid}, UInt16, Ptr{UInt128}, Cuint),
-            sock.handle, hton(port), Ref(hton(host.host)), flags)
-
-_bind(sock::UDPSocket, host::IPv4, port::UInt16, flags::UInt32 = UInt32(0)) = ccall(:jl_udp_bind, Int32, (Ptr{Cvoid}, UInt16, UInt32, UInt32),
-            sock.handle, hton(port), hton(host.host), flags)
-
-_bind(sock::UDPSocket, host::IPv6, port::UInt16, flags::UInt32 = UInt32(0)) = ccall(:jl_udp_bind6, Int32, (Ptr{Cvoid}, UInt16, Ptr{UInt128}, UInt32),
-            sock.handle, hton(port), Ref(hton(host.host)), flags)
+function _bind(sock::UDPSocket, host::Union{IPv4, IPv6}, port::UInt16, flags::UInt32=UInt32(0))
+    host_in = Ref(hton(host.host))
+    return ccall(:jl_udp_bind, Int32, (Ptr{Cvoid}, UInt16, Ptr{Cvoid}, Cuint, Cint),
+            sock, hton(port), host_in, flags, host isa IPv6)
+end
 
 """
     bind(socket::Union{UDPSocket, TCPSocket}, host::IPAddr, port::Integer; ipv6only=false, reuseaddr=false, kws...)
@@ -219,14 +257,16 @@ function bind(sock::Union{TCPServer, UDPSocket}, host::IPAddr, port::Integer; ip
         error("$(typeof(sock)) is not in initialization state")
     end
     flags = 0
-    if isa(host,IPv6) && ipv6only
+    if isa(host, IPv6) && ipv6only
         flags |= isa(sock, UDPSocket) ? UV_UDP_IPV6ONLY : UV_TCP_IPV6ONLY
     end
     if isa(sock, UDPSocket) && reuseaddr
         flags |= UV_UDP_REUSEADDR
     end
+    iolock_begin()
     err = _bind(sock, host, UInt16(port), UInt32(flags))
     if err < 0
+        iolock_end()
         if err != UV_EADDRINUSE && err != UV_EACCES && err != UV_EADDRNOTAVAIL
             #TODO: this codepath is not currently tested
             throw(_UVError("bind", err))
@@ -236,13 +276,14 @@ function bind(sock::Union{TCPServer, UDPSocket}, host::IPAddr, port::Integer; ip
     end
     sock.status = StatusOpen
     isa(sock, UDPSocket) && setopt(sock; kws...)
+    iolock_end()
     return true
 end
 
 bind(sock::TCPServer, addr::InetAddr) = bind(sock, addr.host, addr.port)
 
 """
-    setopt(sock::UDPSocket; multicast_loop = nothing, multicast_ttl=nothing, enable_broadcast=nothing, ttl=nothing)
+    setopt(sock::UDPSocket; multicast_loop=nothing, multicast_ttl=nothing, enable_broadcast=nothing, ttl=nothing)
 
 Set UDP socket options.
 
@@ -252,22 +293,25 @@ Set UDP socket options.
   messages, or else the UDP system will return an access error (default: `false`).
 * `ttl`: Time-to-live of packets sent on the socket (default: `nothing`).
 """
-function setopt(sock::UDPSocket; multicast_loop = nothing, multicast_ttl=nothing, enable_broadcast=nothing, ttl=nothing)
+function setopt(sock::UDPSocket; multicast_loop=nothing, multicast_ttl=nothing, enable_broadcast=nothing, ttl=nothing)
+    iolock_begin()
     if sock.status == StatusUninit
         error("Cannot set options on uninitialized socket")
     end
     if multicast_loop !== nothing
-        uv_error("multicast_loop",ccall(:uv_udp_set_multicast_loop,Cint,(Ptr{Cvoid},Cint),sock.handle,multicast_loop) < 0)
+        uv_error("multicast_loop", ccall(:uv_udp_set_multicast_loop, Cint, (Ptr{Cvoid}, Cint), sock.handle, multicast_loop) < 0)
     end
     if multicast_ttl !== nothing
-        uv_error("multicast_ttl",ccall(:uv_udp_set_multicast_ttl,Cint,(Ptr{Cvoid},Cint),sock.handle,multicast_ttl))
+        uv_error("multicast_ttl", ccall(:uv_udp_set_multicast_ttl, Cint, (Ptr{Cvoid}, Cint), sock.handle, multicast_ttl))
     end
     if enable_broadcast !== nothing
-        uv_error("enable_broadcast",ccall(:uv_udp_set_broadcast,Cint,(Ptr{Cvoid},Cint),sock.handle,enable_broadcast))
+        uv_error("enable_broadcast", ccall(:uv_udp_set_broadcast, Cint, (Ptr{Cvoid}, Cint), sock.handle, enable_broadcast))
     end
     if ttl !== nothing
-        uv_error("ttl",ccall(:uv_udp_set_ttl,Cint,(Ptr{Cvoid},Cint),sock.handle,ttl))
+        uv_error("ttl", ccall(:uv_udp_set_ttl, Cint, (Ptr{Cvoid}, Cint), sock.handle, ttl))
     end
+    iolock_end()
+    nothing
 end
 
 """
@@ -281,88 +325,137 @@ function recv(sock::UDPSocket)
 end
 
 """
-    recvfrom(socket::UDPSocket) -> (address, data)
+    recvfrom(socket::UDPSocket) -> (host_port, data)
 
-Read a UDP packet from the specified socket, returning a tuple of `(address, data)`, where
-`address` will be either IPv4 or IPv6 as appropriate.
+Read a UDP packet from the specified socket, returning a tuple of `(host_port, data)`, where
+`host_port` will be an InetAddr{IPv4} or InetAddr{IPv6}, as appropriate.
+
+!!! compat "Julia 1.3"
+    Prior to Julia version 1.3, the first returned value was an address (`IPAddr`).
+    In version 1.3 it was changed to an `InetAddr`.
 """
 function recvfrom(sock::UDPSocket)
+    iolock_begin()
     # If the socket has not been bound, it will be bound implicitly to ::0 and a random port
     if sock.status != StatusInit && sock.status != StatusOpen && sock.status != StatusActive
         error("UDPSocket is not initialized and open")
     end
     if ccall(:uv_is_active, Cint, (Ptr{Cvoid},), sock.handle) == 0
-        uv_error("recv_start", ccall(:uv_udp_recv_start, Cint, (Ptr{Cvoid}, Ptr{Cvoid}, Ptr{Cvoid}),
-                                    sock.handle, Base.uv_jl_alloc_buf::Ptr{Cvoid}, uv_jl_recvcb::Ptr{Cvoid}))
+        err = ccall(:uv_udp_recv_start, Cint, (Ptr{Cvoid}, Ptr{Cvoid}, Ptr{Cvoid}),
+                    sock, Base.uv_jl_alloc_buf::Ptr{Cvoid}, uv_jl_recvcb::Ptr{Cvoid})
+        uv_error("recv_start", err)
     end
     sock.status = StatusActive
-    return stream_wait(sock, sock.recvnotify)::Tuple{Union{IPv4, IPv6}, Vector{UInt8}}
+    lock(sock.recvnotify)
+    iolock_end()
+    try
+        From = Union{InetAddr{IPv4}, InetAddr{IPv6}}
+        Data = Vector{UInt8}
+        from, data = wait(sock.recvnotify)::Tuple{From, Data}
+        return (from, data)
+    finally
+        unlock(sock.recvnotify)
+    end
 end
 
-alloc_buf_hook(sock::UDPSocket, size::UInt) = (Libc.malloc(size), size)
+alloc_buf_hook(sock::UDPSocket, size::UInt) = (Libc.malloc(size), size) # size is always 64k from libuv
 
 function uv_recvcb(handle::Ptr{Cvoid}, nread::Cssize_t, buf::Ptr{Cvoid}, addr::Ptr{Cvoid}, flags::Cuint)
-    # C signature documented as (*uv_udp_recv_cb)(...)
     sock = @handle_as handle UDPSocket
-    if nread < 0
-        Libc.free(buf_addr)
-        notify_error(sock.recvnotify, _UVError("recv", nread))
-    elseif flags & UV_UDP_PARTIAL > 0
-        Libc.free(buf_addr)
-        notify_error(sock.recvnotify, "Partial message received")
-    else
-        buf_addr = ccall(:jl_uv_buf_base, Ptr{Cvoid}, (Ptr{Cvoid},), buf)
-        buf_size = ccall(:jl_uv_buf_len, Csize_t, (Ptr{Cvoid},), buf)
-        # need to check the address type in order to convert to a Julia IPAddr
-        addrout = if addr == C_NULL
-                      IPv4(0)
-                  elseif ccall(:jl_sockaddr_in_is_ip4, Cint, (Ptr{Cvoid},), addr) == 1
-                      IPv4(ntoh(ccall(:jl_sockaddr_host4, UInt32, (Ptr{Cvoid},), addr)))
-                  else
-                      tmp = [UInt128(0)]
-                      ccall(:jl_sockaddr_host6, UInt32, (Ptr{Cvoid}, Ptr{UInt8}), addr, pointer(tmp))
-                      IPv6(ntoh(tmp[1]))
-                  end
-        buf = unsafe_wrap(Array, convert(Ptr{UInt8}, buf_addr), Int(nread), own = true)
-        notify(sock.recvnotify, (addrout, buf))
+    lock(sock.recvnotify)
+    try
+        buf_addr = ccall(:jl_uv_buf_base, Ptr{UInt8}, (Ptr{Cvoid},), buf)
+        if nread == 0 && addr == C_NULL
+            Libc.free(buf_addr)
+        elseif nread < 0
+            Libc.free(buf_addr)
+            notify_error(sock.recvnotify, _UVError("recv", nread))
+        elseif flags & UV_UDP_PARTIAL > 0
+            Libc.free(buf_addr)
+            notify_error(sock.recvnotify, "Partial message received")
+        else
+            buf_size = Int(ccall(:jl_uv_buf_len, Csize_t, (Ptr{Cvoid},), buf))
+            if buf_size - nread < 16384 # waste at most 16k (note: buf_size is currently always 64k)
+                buf = unsafe_wrap(Array, buf_addr, nread, own=true)
+            else
+                buf = Vector{UInt8}(undef, nread)
+                GC.@preserve buf unsafe_copyto!(pointer(buf), buf_addr, nread)
+                Libc.free(buf_addr)
+            end
+            # need to check the address type in order to convert to a Julia IPAddr
+            host = IPv4(0)
+            port = UInt16(0)
+            if ccall(:jl_sockaddr_is_ip4, Cint, (Ptr{Cvoid},), addr) == 1
+                host = IPv4(ntoh(ccall(:jl_sockaddr_host4, UInt32, (Ptr{Cvoid},), addr)))
+                port = ntoh(ccall(:jl_sockaddr_port4, UInt16, (Ptr{Cvoid},), addr))
+            elseif ccall(:jl_sockaddr_is_ip6, Cint, (Ptr{Cvoid},), addr) == 1
+                tmp = Ref{UInt128}(0)
+                scope_id = ccall(:jl_sockaddr_host6, UInt32, (Ptr{Cvoid}, Ptr{UInt128}), addr, tmp)
+                host = IPv6(ntoh(tmp[]))
+                port = ntoh(ccall(:jl_sockaddr_port6, UInt16, (Ptr{Cvoid},), addr))
+            end
+            from = InetAddr(host, port)
+            notify(sock.recvnotify, (from, buf), all=false)
+        end
+        if sock.status == StatusActive && isempty(sock.recvnotify)
+            sock.status = StatusOpen
+            ccall(:uv_udp_recv_stop, Cint, (Ptr{Cvoid},), sock)
+        end
+    finally
+        unlock(sock.recvnotify)
     end
-    ccall(:uv_udp_recv_stop, Cint, (Ptr{Cvoid},), sock.handle)
-    sock.status = StatusOpen
     nothing
 end
 
-function _send(sock::UDPSocket, ipaddr::IPv4, port::UInt16, buf)
-    ccall(:jl_udp_send, Cint, (Ptr{Cvoid}, UInt16, UInt32, Ptr{UInt8}, Csize_t, Ptr{Cvoid}),
-          sock.handle, hton(port), hton(ipaddr.host), buf, sizeof(buf), uv_jl_sendcb::Ptr{Cvoid})
-end
-
-function _send(sock::UDPSocket, ipaddr::IPv6, port::UInt16, buf)
-    ccall(:jl_udp_send6, Cint, (Ptr{Cvoid}, UInt16, Ref{UInt128}, Ptr{UInt8}, Csize_t, Ptr{Cvoid}),
-          sock.handle, hton(port), hton(ipaddr.host), buf, sizeof(buf), uv_jl_sendcb::Ptr{Cvoid})
+function _send_async(sock::UDPSocket, ipaddr::Union{IPv4, IPv6}, port::UInt16, buf)
+    req = Libc.malloc(Base._sizeof_uv_udp_send)
+    uv_req_set_data(req, C_NULL) # in case we get interrupted before arriving at the wait call
+    host_in = Ref(hton(ipaddr.host))
+    err = ccall(:jl_udp_send, Cint, (Ptr{Cvoid}, Ptr{Cvoid}, UInt16, Ptr{Cvoid}, Ptr{UInt8}, Csize_t, Ptr{Cvoid}, Cint),
+            req, sock, hton(port), host_in, buf, sizeof(buf), Base.uv_jl_writecb_task::Ptr{Cvoid}, ipaddr isa IPv6)
+    if err < 0
+        Libc.free(req)
+        uv_error("send", err)
+    end
+    return req
 end
 
 """
-    send(socket::UDPSocket, host, port::Integer, msg)
+    send(socket::UDPSocket, host::IPAddr, port::Integer, msg)
 
 Send `msg` over `socket` to `host:port`.
 """
-function send(sock::UDPSocket,ipaddr,port,msg)
+function send(sock::UDPSocket, ipaddr::IPAddr, port::Integer, msg)
     # If the socket has not been bound, it will be bound implicitly to ::0 and a random port
+    iolock_begin()
     if sock.status != StatusInit && sock.status != StatusOpen && sock.status != StatusActive
         error("UDPSocket is not initialized and open")
     end
-    uv_error("send", _send(sock, ipaddr, UInt16(port), msg))
-    stream_wait(sock, sock.sendnotify)
-    nothing
-end
-
-function uv_sendcb(handle::Ptr{Cvoid}, status::Cint)
-    sock = @handle_as handle UDPSocket
-    if status < 0
-        notify_error(sock.sendnotify, _UVError("UDP send failed", status))
+    uvw = _send_async(sock, ipaddr, UInt16(port), msg)
+    ct = current_task()
+    preserve_handle(ct)
+    Base.sigatomic_begin()
+    uv_req_set_data(uvw, ct)
+    iolock_end()
+    status = try
+        Base.sigatomic_end()
+        wait()::Cint
+    finally
+        Base.sigatomic_end()
+        iolock_begin()
+        ct.queue === nothing || list_deletefirst!(ct.queue, ct)
+        if uv_req_data(uvw) != C_NULL
+            # uvw is still alive,
+            # so make sure we won't get spurious notifications later
+            uv_req_set_data(uvw, C_NULL)
+        else
+            # done with uvw
+            Libc.free(uvw)
+        end
+        iolock_end()
+        unpreserve_handle(ct)
     end
-    notify(sock.sendnotify)
-    Libc.free(handle)
+    uv_error("send", status)
     nothing
 end
 
@@ -371,48 +464,68 @@ end
 function uv_connectcb(conn::Ptr{Cvoid}, status::Cint)
     hand = ccall(:jl_uv_connect_handle, Ptr{Cvoid}, (Ptr{Cvoid},), conn)
     sock = @handle_as hand LibuvStream
-    if status >= 0
-        if !(sock.status == StatusClosed || sock.status == StatusClosing)
-            sock.status = StatusOpen
+    lock(sock.cond)
+    try
+        if status >= 0 # success
+            if !(sock.status == StatusClosed || sock.status == StatusClosing)
+                sock.status = StatusOpen
+            end
+        else
+            sock.readerror = _UVError("connect", status) # TODO: perhaps we should not reuse readerror for this
+            if !(sock.status == StatusClosed || sock.status == StatusClosing)
+                ccall(:jl_forceclose_uv, Cvoid, (Ptr{Cvoid},), hand)
+                sock.status = StatusClosing
+            end
         end
-        notify(sock.connectnotify)
-    else
-        ccall(:jl_forceclose_uv, Cvoid, (Ptr{Cvoid},), hand)
-        err = _UVError("connect", status)
-        notify_error(sock.connectnotify, err)
+        notify(sock.cond)
+    finally
+        unlock(sock.cond)
     end
     Libc.free(conn)
     nothing
 end
 
-function connect!(sock::TCPSocket, host::IPv4, port::Integer)
+function connect!(sock::TCPSocket, host::Union{IPv4, IPv6}, port::Integer)
+    iolock_begin()
     if sock.status != StatusInit
         error("TCPSocket is not in initialization state")
     end
     if !(0 <= port <= typemax(UInt16))
         throw(ArgumentError("port out of range, must be 0 ≤ port ≤ 65535, got $port"))
     end
-    uv_error("connect", ccall(:jl_tcp4_connect, Int32, (Ptr{Cvoid}, UInt32, UInt16, Ptr{Cvoid}),
-                             sock.handle, hton(host.host), hton(UInt16(port)), uv_jl_connectcb::Ptr{Cvoid}))
-    sock.status = StatusConnecting
-    nothing
-end
-
-function connect!(sock::TCPSocket, host::IPv6, port::Integer)
-    if sock.status != StatusInit
-        error("TCPSocket is not in initialization state")
-    end
-    if !(0 <= port <= typemax(UInt16))
-        throw(ArgumentError("port out of range, must be 0 ≤ port ≤ 65535, got $port"))
-    end
-    uv_error("connect", ccall(:jl_tcp6_connect, Int32, (Ptr{Cvoid}, Ref{UInt128}, UInt16, Ptr{Cvoid}),
-                              sock.handle, hton(host.host), hton(UInt16(port)), uv_jl_connectcb::Ptr{Cvoid}))
+    host_in = Ref(hton(host.host))
+    uv_error("connect", ccall(:jl_tcp_connect, Int32, (Ptr{Cvoid}, Ptr{Cvoid}, UInt16, Ptr{Cvoid}, Cint),
+                              sock, host_in, hton(UInt16(port)), uv_jl_connectcb::Ptr{Cvoid}, host isa IPv6))
     sock.status = StatusConnecting
+    iolock_end()
     nothing
 end
 
 connect!(sock::TCPSocket, addr::InetAddr) = connect!(sock, addr.host, addr.port)
 
+function wait_connected(x::LibuvStream)
+    iolock_begin()
+    check_open(x)
+    isopen(x) || x.readerror === nothing || throw(x.readerror)
+    preserve_handle(x)
+    lock(x.cond)
+    try
+        while x.status == StatusConnecting
+            iolock_end()
+            wait(x.cond)
+            unlock(x.cond)
+            iolock_begin()
+            lock(x.cond)
+        end
+        isopen(x) || x.readerror === nothing || throw(x.readerror)
+    finally
+        unlock(x.cond)
+        unpreserve_handle(x)
+    end
+    iolock_end()
+    nothing
+end
+
 # Default Host to localhost
 
 """
@@ -433,9 +546,7 @@ function connect!(sock::TCPSocket, host::AbstractString, port::Integer)
         error("TCPSocket is not in initialization state")
     end
     ipaddr = getaddrinfo(host)
-    sock.status = StatusInit
-    connect!(sock,ipaddr,port)
-    sock.status = StatusConnecting
+    connect!(sock, ipaddr, port)
     return sock
 end
 
@@ -445,6 +556,40 @@ function connect(sock::LibuvStream, args...)
     return sock
 end
 
+"""
+    nagle(socket::Union{TCPServer, TCPSocket}, enable::Bool)
+
+Enables or disables Nagle's algorithm on a given TCP server or socket.
+"""
+function nagle(sock::Union{TCPServer, TCPSocket}, enable::Bool)
+    # disable or enable Nagle's algorithm on all OSes
+    Sockets.iolock_begin()
+    Sockets.check_open(sock)
+    err = ccall(:uv_tcp_nodelay, Cint, (Ptr{Cvoid}, Cint), sock.handle, Cint(!enable))
+    # TODO: check err
+    Sockets.iolock_end()
+    return err
+end
+
+"""
+    quickack(socket::Union{TCPServer, TCPSocket}, enable::Bool)
+
+On Linux systems, the TCP_QUICKACK is disabled or enabled on `socket`.
+"""
+function quickack(sock::Union{TCPServer, TCPSocket}, enable::Bool)
+    Sockets.iolock_begin()
+    Sockets.check_open(sock)
+    @static if Sys.islinux()
+        # tcp_quickack is a linux only option
+        if ccall(:jl_tcp_quickack, Cint, (Ptr{Cvoid}, Cint), sock.handle, Cint(enable)) < 0
+            @warn "Networking unoptimized ( Error enabling TCP_QUICKACK : $(Libc.strerror(Libc.errno())) )" maxlog=1
+        end
+    end
+    Sockets.iolock_end()
+    nothing
+end
+
+
 ##
 
 const BACKLOG_DEFAULT = 511
@@ -461,31 +606,13 @@ reject them. The default value of `backlog` is 511.
 """
 function listen(addr; backlog::Integer=BACKLOG_DEFAULT)
     sock = TCPServer()
-    !bind(sock, addr) && error("cannot bind to port; may already be in use or access denied")
+    bind(sock, addr) || error("cannot bind to port; may already be in use or access denied")
     listen(sock; backlog=backlog)
     return sock
 end
 listen(port::Integer; backlog::Integer=BACKLOG_DEFAULT) = listen(localhost, port; backlog=backlog)
 listen(host::IPAddr, port::Integer; backlog::Integer=BACKLOG_DEFAULT) = listen(InetAddr(host, port); backlog=backlog)
 
-function listen(callback, server::Union{TCPSocket, UDPSocket})
-    @async begin
-        local client = TCPSocket()
-        while isopen(server)
-            err = accept_nonblock(server, client)
-            if err == 0
-                callback(client)
-                client = TCPSocket()
-            elseif err != UV_EAGAIN
-                uv_error("accept", err)
-            else
-                stream_wait(server, server.connectnotify)
-            end
-        end
-    end
-    return sock
-end
-
 function listen(sock::LibuvServer; backlog::Integer=BACKLOG_DEFAULT)
     uv_error("listen", trylisten(sock))
     return sock
@@ -494,25 +621,33 @@ end
 # from `listen`
 function uv_connectioncb(stream::Ptr{Cvoid}, status::Cint)
     sock = @handle_as stream LibuvServer
-    if status >= 0
-        notify(sock.connectnotify)
-    else
-        notify_error(sock.connectnotify, _UVError("connection", status))
+    lock(sock.cond)
+    try
+        if status >= 0
+            notify(sock.cond)
+        else
+            notify_error(sock.cond, _UVError("connection", status))
+        end
+    finally
+        unlock(sock.cond)
     end
     nothing
 end
 
 function trylisten(sock::LibuvServer; backlog::Integer=BACKLOG_DEFAULT)
+    iolock_begin()
     check_open(sock)
     err = ccall(:uv_listen, Cint, (Ptr{Cvoid}, Cint, Ptr{Cvoid}),
                 sock, backlog, uv_jl_connectioncb::Ptr{Cvoid})
     sock.status = StatusActive
+    iolock_end()
     return err
 end
 
 ##
 
 function accept_nonblock(server::TCPServer, client::TCPSocket)
+    iolock_begin()
     if client.status != StatusInit
         error("client TCPSocket is not in initialization state")
     end
@@ -520,6 +655,7 @@ function accept_nonblock(server::TCPServer, client::TCPSocket)
     if err == 0
         client.status = StatusOpen
     end
+    iolock_end()
     return err
 end
 
@@ -530,19 +666,31 @@ function accept_nonblock(server::TCPServer)
 end
 
 function accept(server::LibuvServer, client::LibuvStream)
-    if server.status != StatusActive
+    iolock_begin()
+    if server.status != StatusActive && server.status != StatusClosing && server.status != StatusClosed
         throw(ArgumentError("server not connected, make sure \"listen\" has been called"))
     end
     while isopen(server)
         err = accept_nonblock(server, client)
         if err == 0
+            iolock_end()
             return client
         elseif err != UV_EAGAIN
             uv_error("accept", err)
         end
-        stream_wait(server, server.connectnotify)
+        preserve_handle(server)
+        lock(server.cond)
+        iolock_end()
+        try
+            wait(server.cond)
+        finally
+            unlock(server.cond)
+            unpreserve_handle(server)
+        end
+        iolock_begin()
     end
     uv_error("accept", UV_ECONNABORTED)
+    nothing
 end
 
 ## Utility functions
@@ -597,6 +745,7 @@ function _sockname(sock, self=true)
     raddress = zeros(UInt8, 16)
     rfamily = Ref{Cuint}(0)
 
+    iolock_begin()
     if self
         r = ccall(:jl_tcp_getsockname, Int32,
                 (Ptr{Cvoid}, Ref{Cushort}, Ptr{Cvoid}, Ref{Cuint}),
@@ -606,33 +755,30 @@ function _sockname(sock, self=true)
                 (Ptr{Cvoid}, Ref{Cushort}, Ptr{Cvoid}, Ref{Cuint}),
                 sock.handle, rport, raddress, rfamily)
     end
+    iolock_end()
     uv_error("cannot obtain socket name", r)
-    if r == 0
-        port = ntoh(rport[])
-        af_inet6 = @static if Sys.iswindows() # AF_INET6 in <sys/socket.h>
-            23
-        elseif Sys.isapple()
-            30
-        elseif Sys.KERNEL ∈ (:FreeBSD, :DragonFly)
-            28
-        elseif Sys.KERNEL ∈ (:NetBSD, :OpenBSD)
-            24
-        else
-            10
-        end
-
-        if rfamily[] == 2 # AF_INET
-            addrv4 = raddress[1:4]
-            naddr = ntoh(unsafe_load(Ptr{Cuint}(pointer(addrv4)), 1))
-            addr = IPv4(naddr)
-        elseif rfamily[] == af_inet6
-            naddr = ntoh(unsafe_load(Ptr{UInt128}(pointer(raddress)), 1))
-            addr = IPv6(naddr)
-        else
-            error(string("unsupported address family: ", getindex(rfamily)))
-        end
+    port = ntoh(rport[])
+    af_inet6 = @static if Sys.iswindows() # AF_INET6 in <sys/socket.h>
+        23
+    elseif Sys.isapple()
+        30
+    elseif Sys.KERNEL ∈ (:FreeBSD, :DragonFly)
+        28
+    elseif Sys.KERNEL ∈ (:NetBSD, :OpenBSD)
+        24
     else
-        error("cannot obtain socket name")
+        10
+    end
+
+    if rfamily[] == 2 # AF_INET
+        addrv4 = raddress[1:4]
+        naddr = ntoh(unsafe_load(Ptr{Cuint}(pointer(addrv4)), 1))
+        addr = IPv4(naddr)
+    elseif rfamily[] == af_inet6
+        naddr = ntoh(unsafe_load(Ptr{UInt128}(pointer(raddress)), 1))
+        addr = IPv6(naddr)
+    else
+        error(string("unsupported address family: ", rfamily[]))
     end
     return addr, port
 end
@@ -647,7 +793,6 @@ function __init__()
     global uv_jl_getaddrinfocb = @cfunction(uv_getaddrinfocb, Cvoid, (Ptr{Cvoid}, Cint, Ptr{Cvoid}))
     global uv_jl_getnameinfocb = @cfunction(uv_getnameinfocb, Cvoid, (Ptr{Cvoid}, Cint, Cstring, Cstring))
     global uv_jl_recvcb        = @cfunction(uv_recvcb, Cvoid, (Ptr{Cvoid}, Cssize_t, Ptr{Cvoid}, Ptr{Cvoid}, Cuint))
-    global uv_jl_sendcb        = @cfunction(uv_sendcb, Cvoid, (Ptr{Cvoid}, Cint))
     global uv_jl_connectioncb  = @cfunction(uv_connectioncb, Cvoid, (Ptr{Cvoid}, Cint))
     global uv_jl_connectcb     = @cfunction(uv_connectcb, Cvoid, (Ptr{Cvoid}, Cint))
 end
