@@ -73,6 +73,14 @@ allsizes = [((), BitArray{0}), ((v1,), BitVector),
     @test !any(d)
     @test all(b)
     @test sz == size(d)
+    @test !isassigned(a, 0)
+    @test !isassigned(b, 0)
+    for ii in 1:prod(sz)
+        @test isassigned(a, ii)
+        @test isassigned(b, ii)
+    end
+    @test !isassigned(a, length(a) + 1)
+    @test !isassigned(b, length(b) + 1)
 end
 
 
@@ -192,6 +200,12 @@ timesofar("utils")
         end
     end
 
+    @testset "constructor from NTuple" begin
+        for nt in ((true, false, false), NTuple{0,Bool}(), (false,), (true,))
+            @test BitVector(nt) == BitVector(collect(nt))
+        end
+    end
+
     @testset "one" begin
         @test Array(one(BitMatrix(undef, 2,2))) == Matrix(I, 2, 2)
         @test_throws DimensionMismatch one(BitMatrix(undef, 2,3))
@@ -545,9 +559,18 @@ timesofar("indexing")
         b2 = bitrand(m2)
         i1 = Array(b1)
         i2 = Array(b2)
+        # Append from array
         @test isequal(Array(append!(b1, b2)), append!(i1, i2))
         @test isequal(Array(append!(b1, i2)), append!(i1, b2))
         @test bitcheck(b1)
+        # Append from HasLength iterator
+        @test isequal(Array(append!(b1, (v for v in b2))), append!(i1, i2))
+        @test isequal(Array(append!(b1, (v for v in i2))), append!(i1, b2))
+        @test bitcheck(b1)
+        # Append from SizeUnknown iterator
+        @test isequal(Array(append!(b1, (v for v in b2 if true))), append!(i1, i2))
+        @test isequal(Array(append!(b1, (v for v in i2 if true))), append!(i1, b2))
+        @test bitcheck(b1)
     end
 
     for m1 = 0:v1, m2 = [0, 1, 63, 64, 65, 127, 128, 129]
@@ -639,10 +662,17 @@ timesofar("indexing")
         @test bitcheck(b1)
     end
     @test length(b1) == 0
+
     b1 = bitrand(v1)
     @test_throws ArgumentError deleteat!(b1, [1, 1, 2])
     @test_throws BoundsError deleteat!(b1, [1, length(b1)+1])
 
+    @test_throws BoundsError deleteat!(BitVector(), 1)
+    @test_throws BoundsError deleteat!(BitVector(), [1])
+    @test_throws BoundsError deleteat!(BitVector(), [2])
+    @test deleteat!(BitVector(), []) == BitVector()
+    @test deleteat!(BitVector(), Bool[]) == BitVector()
+
     b1 = bitrand(v1)
     i1 = Array(b1)
     for j in [63, 64, 65, 127, 128, 129, 191, 192, 193]
@@ -1325,6 +1355,8 @@ timesofar("reductions")
         b2 = bitrand(l)
         @test map(~, b1) == map(x->~x, b1) == broadcast(~, b1)
         @test map(identity, b1) == map(x->x, b1) == b1
+        @test map(zero, b1) == map(x->false, b1) == falses(l)
+        @test map(one, b1) == map(x->true, b1) == trues(l)
 
         @test map(&, b1, b2) == map((x,y)->x&y, b1, b2) == broadcast(&, b1, b2)
         @test map(|, b1, b2) == map((x,y)->x|y, b1, b2) == broadcast(|, b1, b2)
@@ -1513,6 +1545,8 @@ timesofar("linalg")
     for b1 in [falses(v1), trues(v1),
                BitArray([1,0,1,1,0]),
                BitArray([0,0,1,1,0]),
+               BitArray([1 0; 1 1]),
+               BitArray([0 0; 1 1]),
                bitrand(v1)]
         @check_bit_operation findmin(b1)
         @check_bit_operation findmax(b1)
