@@ -21,7 +21,7 @@ end
 msig = Tuple{typeof(f22938),Int,Int,Int,Int}
 world = typemax(UInt)
 _, msp, m = Base._methods_by_ftype(msig, -1, world)[]
-mi = Core.Compiler.code_for_method(m, msig, msp, world, false)
+mi = Core.Compiler.specialize_method(m, msig, msp, false)
 c0 = Core.Compiler.retrieve_code_info(mi)
 
 @test isempty(Core.Compiler.validate_code(mi))
@@ -83,7 +83,7 @@ end
 
 @testset "SLOTFLAGS_MISMATCH" begin
     c = copy(c0)
-    push!(c.slotnames, :dummy)
+    push!(c.slotflags, 0x00)
     errors = Core.Compiler.validate_code(c)
     @test length(errors) == 1
     @test errors[1].kind === Core.Compiler.SLOTFLAGS_MISMATCH
