@@ -126,9 +126,11 @@ end
 @test_throws ArgumentError("overflow in value \"y\" of Enum EnumOvf") @macrocall(@enum EnumOvf x=typemax(Int32) y)
 
 # test for unique Enum values
-@test_throws ArgumentError("values for Enum Test14 are not unique") @macrocall(@enum(Test14, _zero_Test14, _one_Test14, _two_Test14=0))
+@test_throws ArgumentError("both _two_Test14 and _zero_Test14 have value 0 in Enum Test14; values must be unique") @macrocall(@enum(Test14, _zero_Test14, _one_Test14, _two_Test14=0))
+# and names
+@test_throws ArgumentError("name \"_zero_Test15\" in Enum Test15 is not unique") @macrocall(@enum(Test15, _zero_Test15, _one_Test15, _zero_Test15))
 
-@test repr(apple) == "apple::Fruit = 0"
+@test repr(apple) == "$(@__MODULE__).apple"
 @test string(apple) == "apple"
 
 @test repr("text/plain", Fruit) == "Enum $(string(Fruit)):\napple = 0\norange = 1\nkiwi = 2"
@@ -149,8 +151,12 @@ let b = IOBuffer()
     @test deserialize(b) === apple
 end
 
-@enum UI8::UInt8 ten=0x0A thr=0x03 sevn=0x07
-@test repr("text/plain", UI8) == "Enum $(string(UI8)):\nten = 0x0a\nthr = 0x03\nsevn = 0x07"
+@enum UI8::UInt8 ten=0x0A thr=0x03 sevn=0x07 fiftn=0xF0
+@test repr("text/plain", UI8)   == "Enum $(string(UI8)):\nten = 0x0a\nthr = 0x03\nsevn = 0x07\nfiftn = 0xf0"
+@test repr("text/plain", ten)   == "$(string(ten))::UI8 = 0x0a"
+@test repr("text/plain", thr)   == "$(string(thr))::UI8 = 0x03"
+@test repr("text/plain", sevn)  == "$(string(sevn))::UI8 = 0x07"
+@test repr("text/plain", fiftn) == "$(string(fiftn))::UI8 = 0xf0"
 
 # test block form
 @enum BritishFood begin
@@ -159,3 +165,5 @@ end
     haggis       = 4
 end
 @test Int(haggis) == 4
+
+@test (Vector{Fruit}(undef, 3) .= apple) == [apple, apple, apple]
