@@ -102,25 +102,40 @@ end
 # math functions
 
 @testset "real arithmetic" begin
-    for T in (Float32, Float64, BigFloat)
+    for T in (Float16, Float32, Float64, BigFloat)
         half = 1/convert(T,2)
         third = 1/convert(T,3)
 
         for f in (:+, :-, :abs, :abs2, :conj, :inv, :sign,
                   :acos, :asin, :asinh, :atan, :atanh, :cbrt, :cos, :cosh,
-                  :exp10, :exp2, :exp, :expm1, :log10, :log1p,
-                  :log2, :log, :sin, :sinh, :sqrt, :tan, :tanh)
+                  :exp10, :exp2, :exp, :log10, :log1p,
+                  :log2, :log, :sin, :sinh, :sqrt, :tan, :tanh,
+                  :min, :max)
             @eval begin
                 @test @fastmath($f($half)) ≈ $f($half)
                 @test @fastmath($f($third)) ≈ $f($third)
             end
         end
+        if T != Float16
+            for f in (:expm1,)
+                @eval begin
+                    @test @fastmath($f($half)) ≈ $f($half)
+                    @test @fastmath($f($third)) ≈ $f($third)
+                end
+            end
+        end
         for f in (:acosh,)
             @eval begin
                 @test @fastmath($f(1+$half)) ≈ $f(1+$half)
                 @test @fastmath($f(1+$third)) ≈ $f(1+$third)
             end
         end
+        for f in (:sincos,)
+            @eval begin
+                @test all(@fastmath($f($half)) .≈ $f($half))
+                @test all(@fastmath($f($third)) .≈ $f($third))
+            end
+        end
         for f in (:+, :-, :*, :/, :%, :(==), :!=, :<, :<=, :>, :>=, :^,
                   :atan, :hypot, :max, :min, :log)
             @eval begin
@@ -128,6 +143,14 @@ end
                 @test @fastmath($f($third, $half)) ≈ $f($third, $half)
             end
         end
+
+        # issue 31795
+        for f in (:min, :max)
+            @eval begin
+                @test @fastmath($f($half, $third, 1+$half)) ≈ $f($half, $third, 1+$half)
+            end
+        end
+
         for f in (:minmax,)
             @eval begin
                 @test @fastmath($f($half, $third)[1]) ≈ $f($half, $third)[1]
