@@ -179,20 +179,16 @@ module ObjLoadTest
 end
 
 # Test for proper parenting
-if Base.libllvm_version >= v"3.6" # llvm 3.6 changed the syntax for a gep, so just ignore this test on older versions
-    local foo
-    function foo()
-        # this IR snippet triggers an optimization relying
-        # on the llvmcall function having a parent module
-        Base.llvmcall(
-         """%1 = getelementptr i64, i64* null, i64 1
-            ret void""",
-        Cvoid, Tuple{})
-    end
-    code_llvm(devnull, foo, ())
-else
-    @info "Skipping gep parentage test on llvm < 3.6"
+local foo
+function foo()
+    # this IR snippet triggers an optimization relying
+    # on the llvmcall function having a parent module
+    Base.llvmcall(
+     """%1 = getelementptr i64, i64* null, i64 1
+        ret void""",
+    Cvoid, Tuple{})
 end
+code_llvm(devnull, foo, ())
 
 module CcallableRetTypeTest
     using Base: llvmcall, @ccallable
@@ -210,3 +206,46 @@ module CcallableRetTypeTest
     end
     @test do_the_call() === 42.0
 end
+
+# If this test breaks, you've probably broken Cxx.jl - please check
+module LLVMCallFunctionTest
+    using Base: llvmcall
+    using Test
+
+    function julia_to_llvm(@nospecialize x)
+        isboxed = Ref{UInt8}()
+        ccall(:julia_type_to_llvm,Ptr{Cvoid},(Any,Ref{UInt8}),x,isboxed)
+    end
+    const AnyTy = julia_to_llvm(Any)
+
+    const libllvmcalltest = "libllvmcalltest"
+    const the_f = ccall((:MakeIdentityFunction, libllvmcalltest), Ptr{Cvoid}, (Ptr{Cvoid},), AnyTy)
+
+    @eval really_complicated_identity(x) = llvmcall($(the_f), Any, Tuple{Any}, x)
+
+    mutable struct boxed_struct
+    end
+    let x = boxed_struct()
+        @test really_complicated_identity(x) === x
+    end
+
+    # Define two functions that each compute the address of a dedicated internal global variable.
+    # The names of these globals are the same, so if their linkages are overwritten, then the
+    # linker will merge the globals. Consequently, we can test that linkage is preserved by testing
+    # that the addresses of the globals differ. The next few lines of code do just that.
+    const the_other_f1 = ccall((:MakeLoadGlobalFunction, libllvmcalltest), Ptr{Cvoid}, (Ptr{Cvoid},), AnyTy)
+    const the_other_f2 = ccall((:MakeLoadGlobalFunction, libllvmcalltest), Ptr{Cvoid}, (Ptr{Cvoid},), AnyTy)
+
+    @eval global_value_address1() = llvmcall($(the_other_f1), Int64, Tuple{})
+    @eval global_value_address2() = llvmcall($(the_other_f2), Int64, Tuple{})
+
+    @test global_value_address1() != global_value_address2()
+end
+
+# support for calling external functions
+let
+    f() = ccall("time", llvmcall, Cvoid, (Ptr{Cvoid},), C_NULL)
+    @test_throws ErrorException f()
+    f() = ccall("extern time", llvmcall, Cvoid, (Ptr{Cvoid},), C_NULL)
+    f()
+end
