@@ -4,6 +4,9 @@ using Test
 using Base.Threads
 using Base.Threads: SpinLock
 
+# for cfunction_closure
+include("testenv.jl")
+
 # threading constructs
 
 let a = zeros(Int, 2 * nthreads())
@@ -27,26 +30,30 @@ end
 # parallel loop with parallel atomic addition
 function threaded_loop(a, r, x)
     @threads for i in r
-        a[i] = 1 + atomic_add!(x, 1)
+        j = i - firstindex(r) + 1
+        a[j] = 1 + atomic_add!(x, 1)
     end
 end
 
 function test_threaded_loop_and_atomic_add()
-    x = Atomic()
-    a = zeros(Int,10000)
-    threaded_loop(a,1:10000,x)
-    found = zeros(Bool,10000)
-    was_inorder = true
-    for i=1:length(a)
-        was_inorder &= a[i]==i
-        found[a[i]] = true
-    end
-    @test x[] == 10000
-    # Next test checks that all loop iterations ran,
-    # and were unique (via pigeon-hole principle).
-    @test !(false in found)
-    if was_inorder && nthreads() > 1
-        println(stderr, "Warning: threaded loop executed in order")
+    for r in [1:10000, collect(1:10000), Base.IdentityUnitRange(-500:500), (1,2,3,4,5,6,7,8,9,10)]
+        n = length(r)
+        x = Atomic()
+        a = zeros(Int, n)
+        threaded_loop(a,r,x)
+        found = zeros(Bool,n)
+        was_inorder = true
+        for i=1:length(a)
+            was_inorder &= a[i]==i
+            found[a[i]] = true
+        end
+        @test x[] == n
+        # Next test checks that all loop iterations ran,
+        # and were unique (via pigeon-hole principle).
+        @test !(false in found)
+        if was_inorder && nthreads() > 1
+            println(stderr, "Warning: threaded loop executed in order")
+        end
     end
 end
 
@@ -435,16 +442,7 @@ end
 function test_thread_cfunction()
     # ensure a runtime call to `get_trampoline` will be created
     # TODO: get_trampoline is not thread-safe (as this test shows)
-    function complex_cfunction(a)
-        s = zero(eltype(a))
-        @inbounds @simd for i in a
-            s += muladd(a[i], a[i], -2)
-        end
-        return s
-    end
-    fs = [ let a = zeros(10)
-            () -> complex_cfunction(a)
-        end for i in 1:1000 ]
+    fs = [ Core.Box() for i in 1:1000 ]
     @noinline cf(f) = @cfunction $f Float64 ()
     cfs = Vector{Base.CFunction}(undef, length(fs))
     cf1 = cf(fs[1])
@@ -465,10 +463,12 @@ function test_thread_cfunction()
     end
     @test sum(ok) == 10000
 end
-if nthreads() == 1
-    test_thread_cfunction()
-else
-    @test_broken "cfunction trampoline code not thread-safe"
+if cfunction_closure
+    if nthreads() == 1
+        test_thread_cfunction()
+    else
+        @test_broken "cfunction trampoline code not thread-safe"
+    end
 end
 
 # Compare the two ways of checking if threading is enabled.
@@ -701,3 +701,96 @@ let a = zeros(nthreads())
     _atthreads_with_error(a, false)
     @test a == [1:nthreads();]
 end
+
+try
+    @macroexpand @threads(for i = 1:10, j = 1:10; end)
+catch ex
+    @test ex isa LoadError
+    @test ex.error isa ArgumentError
+end
+
+@testset "@spawn interpolation" begin
+    # Issue #30896: evaluating arguments immediately
+    begin
+        outs = zeros(5)
+        # Use interpolation to fill outs with the values of `i`
+        @sync begin
+            local i = 1
+            while i <= 5
+                Threads.@spawn setindex!(outs, $i, $i)
+                i += 1
+            end
+        end
+        @test outs == 1:5
+    end
+
+    # Test macro parsing for interpolating into Args
+    @test fetch(Threads.@spawn 2+$2) == 4
+    @test fetch(Threads.@spawn Int($(2.0))) == 2
+    a = 2
+    @test fetch(Threads.@spawn *($a,$a)) == a^2
+    # Test macro parsing for interpolating into kwargs
+    @test fetch(Threads.@spawn sort($([3 2; 1 0]), dims=2)) == [2 3; 0 1]
+    @test fetch(Threads.@spawn sort([3 $2; 1 $0]; dims=$2)) == [2 3; 0 1]
+
+    # Test macro parsing supports multiple levels of interpolation
+    @testset "spawn macro multiple levels of interpolation" begin
+        # Use `ch` to synchronize within the tests to run after the local variables are
+        # updated, showcasing the problem and the solution.
+        ch = Channel()   # (This synchronization fixes test failure reported in #34141.)
+
+        @test fetch(Threads.@spawn "$($a)") == "$a"
+        let a = 1
+            # Interpolate the current value of `a` vs the value of `a` in the closure
+            t = Threads.@spawn (take!(ch); :(+($$a, $a, a)))
+            a = 2  # update `a` after spawning, before `t` runs
+            put!(ch, nothing)  # now run t
+            @test fetch(t) == Expr(:call, :+, 1, 2, :a)
+        end
+
+        # Test the difference between different levels of interpolation
+        # Without interpolation, each spawned task sees the last value of `i` (6);
+        # with interpolation, each spawned task has the value of `i` at time of `@spawn`.
+        let
+            oneinterp  = Vector{Any}(undef, 5)
+            twointerps = Vector{Any}(undef, 5)
+            @sync begin
+               local i = 1
+               while i <= 5
+                   Threads.@spawn (take!(ch); setindex!(oneinterp, :($i), $i))
+                   Threads.@spawn (take!(ch); setindex!(twointerps, :($($i)), $i))
+                   i += 1
+               end
+               for _ in 1:10; put!(ch, nothing); end # Now run all the tasks.
+            end
+            # The first definition _didn't_ interpolate i
+            @test oneinterp == fill(6, 5)
+            # The second definition _did_ interpolate i
+            @test twointerps == 1:5
+        end
+    end
+end
+
+@testset "@async interpolation" begin
+    # Args
+    @test fetch(@async 2+$2) == 4
+    @test fetch(@async Int($(2.0))) == 2
+    a = 2
+    @test fetch(@async *($a,$a)) == a^2
+    # kwargs
+    @test fetch(@async sort($([3 2; 1 0]), dims=2)) == [2 3; 0 1]
+    @test fetch(@async sort([3 $2; 1 $0]; dims=$2)) == [2 3; 0 1]
+
+    # Supports multiple levels of interpolation
+    @test fetch(@async :($a)) == a
+    @test fetch(@async :($($a))) == a
+    @test fetch(@async "$($a)") == "$a"
+end
+
+# Issue #34138
+@testset "spawn interpolation: macrocalls" begin
+    x = [reshape(1:4, 2, 2);]
+    @test fetch(Threads.@spawn @. $exp(x)) == @. $exp(x)
+    x = 2
+    @test @eval(fetch(@async 2+$x)) == 4
+end
